// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import "AppDelegate.h"
#import "Component1.h"
#import "Component2.h"
#import "SHPMessageRouter.h"
#import "Object1.h"
#import "Object2.h"
#import "React1.h"
#import "React2.h"

typedef enum _Test{
    TestComponent,
    TestMethodInvocation,
    TestReactiveCocoa
}Test;

@implementation AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    // Specify predefined goal
    // **************************************************
    NSUInteger goal = 50000;
    // **************************************************

    // Specify what should be tested
    // **************************************************
    Test test = TestComponent; //TestComponent, TestMethodInvocation or TestReactiveCocoa
    // **************************************************


    switch (test) {
        case TestComponent: {
            // Testing component system
            // **************************************************
            Component1 *component1 = [[Component1 alloc] initWithGoal:goal];
            Component2 *component2 = [Component2 new];
            component1.inputPort.channel = component2.outputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];
            component1.outputPort.channel = component2.inputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];
            [component1 start];
            // **************************************************

            break;
        };
        case TestMethodInvocation: {
            // Testing method invocation
            // **************************************************
            Object1 *object1 = [[Object1 alloc] initWithGoal:goal];
            Object2 *object2 = [Object2 new];
            object1.object2 = object2;
            object2.object1 = object1;
            [object1 start];
            // **************************************************
            break;
        };
        case TestReactiveCocoa: {
            // Testing reactive cocoa
            // **************************************************
            React1 *react1 = [[React1 alloc] initWithGoal:goal];
            React2 *react2 = [[React2 alloc] initWithReact1:react1];
            react1.react2 = react2;
            [react1 start];
            // **************************************************
            break;
        };
    }

    [self setupWindow];

    return YES;
}

- (void)setupWindow {
    self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    self.window.backgroundColor = [UIColor whiteColor];
    [self.window makeKeyAndVisible];
}



@end