// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <Foundation/Foundation.h>

@class GCDAsyncSocket;
@class SHPSocketConnection;

@protocol SHPSocketConnectionDelegate <NSObject>
@optional
- (void)socketConnection:(SHPSocketConnection *)socket didConnectToHost:(NSString *)host port:(uint16_t)port;
- (void)socketConnection:(SHPSocketConnection *)socket didDisconnectWithError:(NSError *)err;
- (void)socketConnection:(SHPSocketConnection *)socket didFailWithError:(NSError *)err;
- (void)socketConnection:(SHPSocketConnection *)socket didReciveObject:(id)object;

@end

typedef struct {
    UInt16 type;
    UInt32 blockTag;
    UInt32 bodyLength;
} SHPSocketConnectionHeader;

@interface SHPSocketConnection : NSObject <NSNetServiceDelegate>
@property(nonatomic, weak, readonly) id<SHPSocketConnectionDelegate> delegate;
@property(nonatomic, readonly) uint16_t port;


- (id)initWithHost:(NSString *)host port:(uint16_t)port delegate:(id<SHPSocketConnectionDelegate>)delegate;

- (id)initWithSocket:(GCDAsyncSocket *)socket delegate:(id<SHPSocketConnectionDelegate>)delegate;
- (void)connect;
- (void)disconnect;
- (BOOL)connected;
- (void)sendObject:(id <NSCoding>)object;
@end