// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <Foundation/Foundation.h>
#import "SHPStateChart.h"
#import "SHPSocketConnection.h"
#import "SHPInputPort.h"
#import "SHPOutputPort.h"

@class GCDAsyncSocket;
@class SHPSocketConnectionInfo;

@protocol SHPSocketClientCommandProtocol
@optional
- (void)eventSocketClientConnectWithConnectionInfo:(SHPSocketConnectionInfo *)connectionInfo;
- (void)eventSocketClientConnect;
- (void)eventSocketClientDisconnect;
- (void)eventSocketClientSendObject:(id <NSCoding>)object;
@end

@protocol SHPSocketClientNotificationProtocol
@optional
- (void)eventSocketClientConnected;
- (void)eventSocketClientDisconnected;
- (void)eventSocketClientFailedWithError:(NSError *)error;
- (void)eventSocketClientReceivedObject:(id <NSCoding>)object;
@end

@interface SHPSocketClientStateChart : SHPStateChart <SHPSocketConnectionDelegate>
@property(nonatomic) SHPSocketConnection *socketConnection;
@property (nonatomic, readonly) SHPInputPort<SHPSocketClientCommandProtocol> *commandInputPort;
@property(nonatomic, readonly) SHPOutputPort<SHPSocketClientNotificationProtocol> *notificationOutputPort;

- (id)initWithSocket:(GCDAsyncSocket *)socket;
@end