// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import "DebtsModel.h"
#import "ExpenseModel.h"
#import "Debt.h"
#import "User.h"
#import "SHPEventMessage.h"
#import "Expense.h"

BASE_STATE(DebtsModel)

#pragma mark - StateOverall
STATE(DebtsModel, StateOverall)


- (void)fetchedExpenses:(NSArray *)expenses {
    NSMutableArray *debts = [NSMutableArray new];

    NSMutableArray *people = [NSMutableArray new];
    for (Expense *expense in expenses) {
        if(![people containsObject:expense.payer]) {
            [people addObject:expense.payer];
        }
    }

    if(people.count>1) {
        __block CGFloat difference = 0.0f;
        User *primaryUser = people.firstObject;
        User *secondaryUser = people[1];
        [expenses enumerateObjectsUsingBlock:^(Expense *expense, NSUInteger idx, BOOL *stop) {
            if(expense.payer == primaryUser) {
                difference += [expense.amount floatValue]/2.0f;
            } else {
                difference -= [expense.amount floatValue]/2.0f;
            }
        }];

        Debt *debt;
        if(difference>0.0f) {
            debt = [[Debt alloc] initWithTaker:primaryUser giver:secondaryUser amount:@(difference)];
        } else {
            debt = [[Debt alloc] initWithTaker:secondaryUser giver:primaryUser amount:@(-difference)];
        }

        [debts addObject:debt];
    }


    [self.stateChart.outputPort didFetchDebts:debts];
    [self.stateChart.eventPort didFetchDebts:debts];
}

STATE_END

#pragma mark - StateWaiting
STATE(DebtsModel, StateWaiting)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    NSLog(@"StateWaiting");
}

- (void)fetchDebts {
    [self.stateChart.expenseOutputPort fetchExpenses];
}

STATE_END

#pragma mark - StateFetching
STATE(DebtsModel, StateFetching)


STATE_END

@implementation DebtsModel {

}

- (id)init {
    self = [super init];
    if (self) {
        _inputPort = [self inputPortWithProtocol:@protocol(DebtsInputProtocol) name:@"Input"];
        _outputPort = [self outputPortWithProtocol:@protocol(DebtsOutputProtocol) name:@"Output"];

        _expenseInputPort = [self inputPortWithProtocol:@protocol(ExpenseModelOutputProtocol) name:@"Expense input"];
        _expenseOutputPort = [self outputPortWithProtocol:@protocol(ExpenseModelInputProtocol) name:@"Expense output"];
    }

    return self;
}


- (void)configureWithConfiguration:(SHPStateChartConfiguration *)configuration {
    StateOverall *stateOverall = [StateOverall new];
    StateWaiting *stateWaiting = [StateWaiting new];
    StateFetching *stateFetching = [StateFetching new];

    [stateOverall setSubStates:@[stateWaiting, stateFetching] initialState:stateWaiting];
    [configuration.rootState setSubStates:@[stateOverall] initialState:stateOverall];
    [configuration setEventHandlerProtocol:@protocol(DebtsModelEventProtocol)];

    [stateWaiting onEvent:@selector(fetchDebts) transitionTo:stateFetching];
    [stateFetching onEvent:@selector(didFetchDebts:) transitionTo:stateWaiting];
    [stateOverall internalTransitionOnEvent:@selector(fetchedExpenses:)];
}


@end