// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import "ExpenseModel.h"
#import "SHPEventMessage.h"
#import "Expense.h"

BASE_STATE(ExpenseModel)

#pragma mark - StateWorkFlow
AND_STATE(ExpenseModel, StateWorkFlow)STATE_END

#pragma mark - StateFetchExpenses
STATE(ExpenseModel, StateFetchExpenses)STATE_END

#pragma mark - StateIdle
STATE(ExpenseModel, StateIdle)STATE_END

#pragma mark - StateFetching
STATE(ExpenseModel, StateFetching)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.webserviceOutputPort fetchExpenses];
}

- (void)didFetchExpenses:(NSArray *)expenses {
    [self.stateChart.outputPort fetchedExpenses:expenses];
}

STATE_END

#pragma mark - StateDeleteExpense
STATE(ExpenseModel, StateDeleteExpense)STATE_END

#pragma mark - StateDeleteIdle
STATE(ExpenseModel, StateDeleteIdle)

- (void)deleteExpense:(Expense *)expense {
    [self.stateChart.webserviceOutputPort removeExpenseForIdentifier:expense.identifier];
}

STATE_END

#pragma mark - StateDeleting
STATE(ExpenseModel, StateDeleting)

- (void)didRemoveExpenseWithIdentifier:(NSString *)identifier {
    [self.stateChart.eventPort fetchExpenses];
}

STATE_END


#pragma mark - StateCreateExpense
STATE(ExpenseModel, StateCreateExpense)STATE_END

#pragma mark - StateCreatingExpense
STATE(ExpenseModel, StateCreatingExpense)

- (void)didCreateExpenseWithIdentifier:(NSString *)identifier {
    [self.stateChart.eventPort fetchExpenses];
}

STATE_END

#pragma mark - StateCreateIdle
STATE(ExpenseModel, StateCreateIdle)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [super eventEntry:eventMessage];
}


- (void)createExpenseWithAmount:(NSNumber *)amount category:(NSString *)category {
    [self.stateChart.webserviceOutputPort createExpenseWithAmount:amount category:category];
}


STATE_END

@implementation ExpenseModel {

}

- (id)init {
    self = [super init];
    if (self) {
        _inputPort = [self inputPortWithProtocol:@protocol(ExpenseModelInputProtocol) name:@"Input"];
        _outputPort = [self outputPortWithProtocol:@protocol(ExpenseModelOutputProtocol) name:@"Output"];

        _webserviceInputPort = [self inputPortWithProtocol:@protocol(WebserviceOutputProtocol) name:@"Webservice Input"];
        _webserviceOutputPort = [self outputPortWithProtocol:@protocol(WebserviceInputProtocol) name:@"Webservice Output"];
    }

    return self;
}

- (void)configureWithConfiguration:(SHPStateChartConfiguration *)configuration {
    StateWorkFlow *stateWorkFlow = [StateWorkFlow new];
    StateFetchExpenses *stateFetchExpenses = [StateFetchExpenses new];
    StateDeleteExpense *stateDeleteExpense = [StateDeleteExpense new];
    StateIdle *stateIdle = [StateIdle new];
    StateFetching *stateFetching = [StateFetching new];
    StateDeleteIdle *stateDeleteIdle = [StateDeleteIdle new];
    StateDeleting *stateDeleting = [StateDeleting new];

    StateCreateExpense *stateCreateExpense = [StateCreateExpense new];
    StateCreatingExpense *stateCreatingExpense = [StateCreatingExpense new];
    StateCreateIdle *stateCreateIdle = [StateCreateIdle new];

    configuration.eventHandlerProtocol = @protocol(ExpenseModelEventProtocol);
    [configuration.rootState setSubStates:@[stateWorkFlow] initialState:stateWorkFlow];
    [stateWorkFlow setRegions:@[stateFetchExpenses, stateDeleteExpense, stateCreateExpense]];
    [stateFetchExpenses setSubStates:@[stateIdle, stateFetching] initialState:stateIdle];
    [stateDeleteExpense setSubStates:@[stateDeleteIdle, stateDeleting] initialState:stateDeleteIdle];
    [stateCreateExpense setSubStates:@[stateCreateIdle, stateCreatingExpense] initialState:stateCreateIdle];

    [stateIdle onEvent:@selector(fetchExpenses) transitionTo:stateFetching];
    [stateFetching onEvent:@selector(didFetchExpenses:) transitionTo:stateIdle];

    [stateDeleteIdle onEvent:@selector(deleteExpense:) transitionTo:stateDeleting];
    [stateDeleting onEvent:@selector(didRemoveExpenseWithIdentifier:) transitionTo:stateDeleteIdle];

    [stateCreateIdle onEvent:@selector(createExpenseWithAmount:category:) transitionTo:stateCreatingExpense];
    [stateCreatingExpense onEvent:@selector(didCreateExpenseWithIdentifier:) transitionTo:stateCreateIdle];
}

@end