// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import "WebserviceDummyComponent.h"
#import "User.h"
#import "Expense.h"
#import "ExpenseCategory.h"


@implementation WebserviceDummyComponent {
    NSMutableArray *_expenses;
    User *_currentUser;
}

- (id)init {
    self = [super init];
    if (self) {
        _inputPort = [self inputPortWithProtocol:@protocol(WebserviceInputProtocol) name:@"Input"];
        _outputPort = [self outputPortWithProtocol:@protocol(WebserviceOutputProtocol) name:@"Output"];


        _currentUser = [[User alloc] initWithName:@"Peter"];
        User *anotherUser =[[User alloc] initWithName:@"Philip"];
        ExpenseCategory *category1 = [[ExpenseCategory alloc] initWithName:@"Food"];
        ExpenseCategory *category2 = [[ExpenseCategory alloc] initWithName:@"Technology"];
        ExpenseCategory *category3 = [[ExpenseCategory alloc] initWithName:@"Travel"];

        Expense *expense1 = [[Expense alloc] initWithAmount:@250 payer:_currentUser category:category1];
        Expense *expense2 = [[Expense alloc] initWithAmount:@150 payer:anotherUser category:category2];
        Expense *expense3 = [[Expense alloc] initWithAmount:@50 payer:_currentUser category:category2];
        Expense *expense4 = [[Expense alloc] initWithAmount:@20 payer:anotherUser category:category1];
        Expense *expense5 = [[Expense alloc] initWithAmount:@542 payer:_currentUser category:category3];
        Expense *expense6 = [[Expense alloc] initWithAmount:@231 payer:_currentUser category:category1];
        Expense *expense7 = [[Expense alloc] initWithAmount:@653 payer:anotherUser category:category3];
        Expense *expense8 = [[Expense alloc] initWithAmount:@100 payer:anotherUser category:category1];

        _expenses = [@[expense1, expense2, expense3, expense4, expense5, expense6, expense7, expense8] mutableCopy];
    }

        return self;
    }


- (void)fetchExpenses {
        [NSThread sleepForTimeInterval:.5f];

     [self.outputPort didFetchExpenses:_expenses];
}

- (void)removeExpenseForIdentifier:(NSString *)identifier {
    for (NSUInteger i = 0; i < _expenses.count; i++) {
        Expense *expense = _expenses[i];
        if ([identifier isEqualToString:expense.identifier]) {
            [_expenses removeObject:expense];

            [self.outputPort didRemoveExpenseWithIdentifier:identifier];
            return;
        }
    }
}

- (void)createExpenseWithAmount:(NSNumber *)amount category:(NSString *)category {
    Expense *expense = [[Expense alloc] initWithAmount:amount payer:_currentUser category:[[ExpenseCategory alloc] initWithName:category]];
    [_expenses addObject:expense];

    [self.outputPort didCreateExpenseWithIdentifier:expense.identifier];
}

@end