// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <SHPStateChart/SHPProxyOutputPort.h>
#import <SHPStateChart/SHPStateChartGuard.h>
#import "AddExpenseStateChart.h"
#import "AddExpenseLib.h"
#import "ApplicationStateChart.h"
#import "SHPEventMessage.h"
#import "SHPStateChartTransition.h"


BASE_STATE(AddExpenseStateChart)

#pragma mark - StateOverall
AND_STATE(AddExpenseStateChart, StateOverall)

- (void)cancelAddExpensePressed {
    [self.stateChart.lib clearData];
}

- (void)amountTextFieldChangedToAmount:(NSString *)amount {
    self.stateChart.amount = amount;
}

- (void)categoryTextFieldChangedToCategory:(NSString *)category {
    self.stateChart.category = category;
}


STATE_END

#pragma mark - StateAmount
STATE(AddExpenseStateChart, StateAmount)STATE_END

#pragma mark - StateAmountInvalid
STATE(AddExpenseStateChart, StateAmountInvalid)STATE_END

#pragma mark - StateAmountValid
STATE(AddExpenseStateChart, StateAmountValid)STATE_END

#pragma mark - StateCategory
STATE(AddExpenseStateChart, StateCategory)STATE_END

#pragma mark - StateCategoryInvalid
STATE(AddExpenseStateChart, StateCategoryInvalid)STATE_END

#pragma mark - StateCategoryValid
STATE(AddExpenseStateChart, StateCategoryValid)STATE_END

#pragma mark - StateAddButton
STATE(AddExpenseStateChart, StateAddButton)STATE_END

#pragma mark - StateAddButtonDisabled
STATE(AddExpenseStateChart, StateAddButtonDisabled)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.lib setAddButtonEnabled:NO];
}


STATE_END

#pragma mark - StateAddButtonEnabled
STATE(AddExpenseStateChart, StateAddButtonEnabled)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.lib setAddButtonEnabled:YES];
}

- (void)createPressedWithAmount:(NSInteger)amount categoryString:(NSString *)categoryString {
    [self.stateChart.lib clearData];
    [self.stateChart.outputPort createExpenseWithAmount:@(amount) category:categoryString];
}


STATE_END

@implementation AddExpenseStateChart {

}

- (id)initWithRootState:(SHPOrState *)rootState {
    self = [super initWithRootState:rootState];
    if (self) {
        _outputPort = [self proxyOutputPortWithProtocol:@protocol(ApplicationStateChartOutputProtocol) name:@"Output"];
    }

    return self;
}

- (void)configureWithConfiguration:(SHPStateChartConfiguration *)configuration {
    StateOverall *stateOverall = [StateOverall new];
    StateAmount *stateAmount = [StateAmount new];
    StateAmountInvalid *stateAmountInvalid = [StateAmountInvalid new];
    StateAmountValid *stateAmountValid = [StateAmountValid new];
    StateCategory *stateCategory = [StateCategory new];
    StateCategoryInvalid *stateCategoryInvalid = [StateCategoryInvalid new];
    StateCategoryValid *stateCategoryValid = [StateCategoryValid new];
    StateAddButton *stateAddButton = [StateAddButton new];
    StateAddButtonDisabled *stateAddButtonDisabled = [StateAddButtonDisabled new];
    StateAddButtonEnabled *stateAddButtonEnabled = [StateAddButtonEnabled new];

    [stateOverall setRegions:@[stateAmount, stateCategory, stateAddButton]];
    [stateAmount setSubStates:@[stateAmountInvalid, stateAmountValid] initialState:stateAmountInvalid];
    [stateCategory setSubStates:@[stateCategoryInvalid, stateCategoryValid] initialState:stateCategoryInvalid];
    [stateAddButton setSubStates:@[stateAddButtonDisabled, stateAddButtonEnabled] initialState:stateAddButtonDisabled];
    [configuration.rootState setSubStates:@[stateOverall] initialState:stateOverall];
    [configuration setEventHandlerProtocol:@protocol(AddExpenseStateChartEventProtocol)];

    [stateOverall internalTransitionOnEvent:@selector(cancelAddExpensePressed)];
    [stateAddButtonEnabled internalTransitionOnEvent:@selector(createPressedWithAmount:categoryString:)];


    [stateAmountInvalid transitionTo:stateAmountValid guard:[[SHPStateChartGuard alloc] initWithCondition:^BOOL{
        if (!self.amount) return NO;

        BOOL valid;
        NSCharacterSet *alphaNums = [NSCharacterSet decimalDigitCharacterSet];
        NSCharacterSet *inStringSet = [NSCharacterSet characterSetWithCharactersInString:self.amount];
        valid = [alphaNums isSupersetOfSet:inStringSet];

        if (valid) {
            NSInteger amount = [_amount intValue];
            return amount>0;
        }

        return NO;
    } guardDescription:@"amount > 0"] reverse:YES];

    [stateCategoryInvalid transitionTo:stateCategoryValid guard:[[SHPStateChartGuard alloc] initWithCondition:^BOOL{
        return _category.length>2;
    } guardDescription:@"Category.length>2"] reverse:YES];

    [stateAddButtonDisabled transitionTo:stateAddButtonEnabled guard:[[SHPStateChartGuard alloc] initWithCondition:^BOOL{
        return stateAmount.activeState == stateAmountValid && stateCategory.activeState == stateCategoryValid;
    } guardDescription:@"amount and category valid"] reverse:YES];

    [stateOverall internalTransitionOnEvent:@selector(amountTextFieldChangedToAmount:)];
    [stateOverall internalTransitionOnEvent:@selector(categoryTextFieldChangedToCategory:)];
}


@end