// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <UIKit/UIKit.h>
#import <SHPStateChart/SHPProxyOutputPort.h>
#import "ApplicationStateChart.h"
#import "SHPEventMessage.h"
#import "SHPStateChartGuard.h"
#import "ExpenseModel.h"
#import "TabBarStateChart.h"
#import "SHPProxyPort.h"
#import "SHPProxyOutputPort.h"
#import "SHPMessageRouter.h"
#import "ModalStateChart.h"
#import "DebtsStateChart.h"

BASE_STATE(ApplicationStateChart)

#pragma mark - StateOverall
AND_STATE(ApplicationStateChart, StateOverall)STATE_END

#pragma mark - StateTabBar
STATE(ApplicationStateChart, StateTabBar)STATE_END

#pragma mark - StateTabController
AND_STATE(ApplicationStateChart, StateTabController)STATE_END

#pragma mark - StateExpenses
STATE(ApplicationStateChart, StateExpenses)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    NSLog(@"Entry expenses");
}

STATE_END

#pragma mark - StateDebts
STATE(ApplicationStateChart, StateDebts)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    NSLog(@"Entry debts");
}

STATE_END

#pragma mark - StateHelper
STATE(ApplicationStateChart, StateHelper)STATE_END

#pragma mark - StateHelperWaiting
STATE(ApplicationStateChart, StateHelperWaiting)STATE_END

#pragma mark - StateHelperReady
STATE(ApplicationStateChart, StateHelperReady)

NSArray *_expenses;

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.eventPort refreshExpenses];
}

- (void)fetchedExpenses:(NSArray *)expenses {
    _expenses = expenses;
}

STATE_END

#pragma mark - StateHelperFetching
STATE(ApplicationStateChart, StateHelperFetched)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.eventPort detailPressedForExpense:_expenses[0]];
}


STATE_END



@implementation ApplicationStateChart {

}

- (id)init {
    _outputPort = [self proxyOutputPortWithProtocol:@protocol(ApplicationStateChartOutputProtocol) name:@"Output"];

    self = [super init];
    if (self) {
        _inputPort = [self inputPortWithProtocol:@protocol(ApplicationStateChartInputProtocol) name:@"Input"];
        _dataInputPort = [self inputPortWithProtocol:@protocol(ApplicationStateChartDataInputProtocol) name:@"Data input"];
    }

    return self;
}

- (void)configureWithConfiguration:(SHPStateChartConfiguration *)configuration {
    StateOverall *stateOverall = [StateOverall new];
    StateTabBar *stateTabBar = [StateTabBar new];
    StateTabController *stateTabController = [StateTabController new];
    StateExpenses *stateExpenses = [StateExpenses new];
    StateDebts *stateDebts = [StateDebts new];


    _expensesNavigationControllerStateChart = [[ExpensesNavigationControllerStateChart alloc] initWithRootState:stateExpenses];
    _debtsStateChart = [[DebtsStateChart alloc] initWithRootState:stateDebts];
    _tabBarStateChart = [[TabBarStateChart alloc] initWithRootState:stateTabBar];

    _outputPort.proxyChannel = _expensesNavigationControllerStateChart.outputPort.channel = _tabBarStateChart.outputPort.channel = _debtsStateChart.outputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];

    //Helper state
    StateHelper *stateHelper = [StateHelper new];
    StateHelperWaiting *stateHelperWaiting = [StateHelperWaiting new];
    StateHelperReady *stateHelperReady = [StateHelperReady new];
    StateHelperFetched *stateHelperFetched = [StateHelperFetched new];
    [stateHelper setSubStates:@[stateHelperWaiting, stateHelperReady, stateHelperFetched] initialState:stateHelperWaiting];

//    SHPOrState *expensesListOverall = (SHPOrState *) [_expensesNavigationControllerStateChart.expensesListStateChart findStateOfClass:NSClassFromString(@"ExpensesListStateChartStateOverall")];
//    SHPOrState *expensesListWaiting = (SHPOrState *) [_expensesNavigationControllerStateChart.expensesListStateChart findStateOfClass:NSClassFromString(@"ExpensesListStateChartStateWaiting")];
//    SHPOrState *expensesListShowingData = (SHPOrState *) [_expensesNavigationControllerStateChart.expensesListStateChart findStateOfClass:NSClassFromString(@"ExpensesListStateChartStateShowingData")];
//
//    [stateHelperWaiting transitionTo:stateHelperReady guard:[[SHPStateChartGuard alloc] initWithCondition:^BOOL{
//        return expensesListOverall.activeState == expensesListWaiting;
//    } description:@""]];
//
//    [stateHelperReady internalTransitionOnEvent:@selector(fetchedExpenses:)];
//
//    [stateHelperReady transitionTo:stateHelperFetched guard:[[SHPStateChartGuard alloc] initWithCondition:^BOOL{
//        return expensesListOverall.activeState == expensesListShowingData;
//    } description:@""]];


    [stateOverall setRegions:@[stateTabBar, stateTabController, stateHelper]];
    [stateTabController setRegions:@[stateExpenses, stateDebts]];

    [configuration.rootState setSubStates:@[stateOverall] initialState:stateOverall];
    [configuration setEventHandlerProtocol:@protocol(ApplicationStateChartEventProtocol)];
}

@end