// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <SHPStateChart/SHPProxyOutputPort.h>
#import <SHPStateChart/SHPEventMessage.h>
#import "DebtsStateChart.h"
#import "ApplicationStateChart.h"
#import "DebtsLib.h"

BASE_STATE(DebtsStateChart)

#pragma mark - StateOverall
STATE(DebtsStateChart, StateOverall)

- (void)didFetchDebts:(NSArray *)debts {
    [self.stateChart.lib showDebts:debts];
}

STATE_END

#pragma mark - StateWaiting
STATE(DebtsStateChart, StateWaiting)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.lib showDebts:@[]];
}


- (void)refreshDebts {
    [self.stateChart.outputPort fetchDebts];
}

STATE_END

#pragma mark - StateLoading
STATE(DebtsStateChart, StateLoading)


STATE_END

#pragma mark - StateShowingData
STATE(DebtsStateChart, StateShowingData)

- (void)refreshDebts {
    [self.stateChart.outputPort fetchDebts];
}

STATE_END


@implementation DebtsStateChart {

}


- (id)initWithRootState:(SHPOrState *)rootState {
    self = [super initWithRootState:rootState];
    if (self) {
        _outputPort = [self proxyOutputPortWithProtocol:@protocol(ApplicationStateChartOutputProtocol) name:@"Output"];
    }

    return self;
}

- (void)configureWithConfiguration:(SHPStateChartConfiguration *)configuration {
    StateOverall *stateOverall = [StateOverall new];
    StateWaiting *stateWaiting = [StateWaiting new];
    StateLoading *stateLoading = [StateLoading new];
    StateShowingData *stateShowingData = [StateShowingData new];

    [configuration setEventHandlerProtocol:@protocol(ExpenseModelEventProtocol)];
    [configuration.rootState setSubStates:@[stateOverall] initialState:stateOverall];
    [stateOverall setSubStates:@[stateWaiting, stateLoading, stateShowingData] initialState:stateWaiting];

    [stateOverall onEvent:@selector(didFetchDebts:) transitionTo:stateShowingData];
    [stateWaiting onEvent:@selector(refreshDebts) transitionTo:stateLoading];
    [stateShowingData onEvent:@selector(refreshDebts) transitionTo:stateLoading];
    [stateShowingData onEvent:@selector(logout) transitionTo:stateWaiting];
}
@end