// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <SHPStateChart/SHPProxyOutputPort.h>
#import "ExpenseDetailStateChart.h"
#import "SHPEventMessage.h"
#import "ExpenseDetailLib.h"
#import "Expense.h"
#import "ApplicationStateChart.h"


BASE_STATE(ExpenseDetailStateChart)

#pragma mark - StateStarted
STATE(ExpenseDetailStateChart, StateStarted)

- (void)deletePressedForExpense:(Expense *)expense {
    [self.stateChart.outputPort deleteExpense:expense];
}

- (void)detailPressedForExpense:(Expense *)expense {
    [self.stateChart.lib showExpense:expense];
}

STATE_END

@implementation ExpenseDetailStateChart {

}

- (id)initWithRootState:(SHPOrState *)rootState {
    self = [super initWithRootState:rootState];
    if (self) {
        _outputPort = [self proxyOutputPortWithProtocol:@protocol(ApplicationStateChartOutputProtocol) name:@"Output"];
    }

    return self;
}

- (void)configureWithConfiguration:(SHPStateChartConfiguration *)configuration {
    StateStarted *stateStarted = [StateStarted new];

    [configuration.rootState setSubStates:@[stateStarted] initialState:stateStarted];
    [configuration setEventHandlerProtocol:@protocol(ExpenseDetailStateChartEventProtocol)];

    [stateStarted internalTransitionOnEvent:@selector(detailPressedForExpense:)];
    [stateStarted internalTransitionOnEvent:@selector(deletePressedForExpense:)];

}


@end