// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import "ExpensesListStateChart.h"
#import "ExpensesListLib.h"
#import "SHPProxyOutputPort.h"
#import "SHPEventMessage.h"
#import "ApplicationStateChart.h"

BASE_STATE(ExpensesListStateChart)

#pragma mark - StateOverall
STATE(ExpensesListStateChart, StateOverall)

- (void)fetchedExpenses:(NSArray *)expenses {
    [self.stateChart.lib showExpenses:expenses];
}

STATE_END

#pragma mark - StateWaiting
STATE(ExpensesListStateChart, StateWaiting)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.lib showExpenses:@[]];
}

- (void)refreshExpenses {
    [self.stateChart.outputPort fetchExpenses];
}

STATE_END

#pragma mark - StateLoading
STATE(ExpensesListStateChart, StateLoading)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    NSLog(@"Loading data");
}

STATE_END

#pragma mark - StateShowingData
STATE(ExpensesListStateChart, StateShowingData)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    NSLog(@"Showind data");
}

- (void)refreshExpenses {
    [self.stateChart.outputPort fetchExpenses];
}

STATE_END

@implementation ExpensesListStateChart {

}

- (id)initWithRootState:(SHPOrState *)rootState {
    self = [super initWithRootState:rootState];
    if (self) {
        _outputPort = [self proxyOutputPortWithProtocol:@protocol(ApplicationStateChartOutputProtocol) name:@"Output"];
    }

    return self;
}

- (void)configureWithConfiguration:(SHPStateChartConfiguration *)configuration {
    StateOverall *stateOverall = [StateOverall new];
    StateWaiting *stateWaiting = [StateWaiting new];
    StateLoading *stateLoading = [StateLoading new];
    StateShowingData *stateShowingData = [StateShowingData new];

    [configuration setEventHandlerProtocol:@protocol(ExpenseModelEventProtocol)];
    [configuration.rootState setSubStates:@[stateOverall] initialState:stateOverall];
    [stateOverall setSubStates:@[stateWaiting, stateLoading, stateShowingData] initialState:stateWaiting];

    [stateOverall onEvent:@selector(fetchedExpenses:) transitionTo:stateShowingData];
    [stateWaiting onEvent:@selector(refreshExpenses) transitionTo:stateLoading];
    [stateShowingData onEvent:@selector(refreshExpenses) transitionTo:stateLoading];
    [stateShowingData onEvent:@selector(logout) transitionTo:stateWaiting];
}

@end