// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <SHPStateChart/SHPProxyOutputPort.h>
#import "ExpensesNavigationControllerStateChart.h"
#import "SHPEventMessage.h"
#import "ExpensesNavigationBarLib.h"
#import "ExpensesListStateChart.h"
#import "SHPProxyPort.h"
#import "SHPProxyOutputPort.h"
#import "ModelComponent.h"
#import "SHPMessageRouter.h"
#import "ApplicationStateChart.h"
#import "Expense.h"
#import "ExpenseDetailViewController.h"
#import "ExpenseDetailStateChart.h"

BASE_STATE(ExpensesNavigationControllerStateChart)

#pragma mark - StateOverall
AND_STATE(ExpensesNavigationControllerStateChart, StateOverall)STATE_END

#pragma mark - StateExpensesList
STATE(ExpensesNavigationControllerStateChart, StateExpensesList)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    NSLog(@"Entry expenses list");
}

STATE_END

#pragma mark - StateExpenseDetail
STATE(ExpensesNavigationControllerStateChart, StateExpenseDetail)STATE_END

#pragma mark - StateNavigationBar
STATE(ExpensesNavigationControllerStateChart, StateNavigationBar)STATE_END

#pragma mark - StateRootViewController
STATE(ExpensesNavigationControllerStateChart, StateRootViewController)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    NSLog(@"Entry root viewcontroller");
}

- (void)detailPressedForExpense:(Expense *)expense {
    [self.stateChart.lib transitionToViewControllerAtIndex:1];
}

STATE_END

#pragma mark - StateDetailViewController
STATE(ExpensesNavigationControllerStateChart, StateDetailViewController)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    NSLog(@"Entry DetailViewController");
}

- (void)logout {
    [self.stateChart.lib popToRoot];
}

STATE_END

#pragma mark - StateRootToDetailAnimation
STATE(ExpensesNavigationControllerStateChart, StateRootToDetailAnimation)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    NSLog(@"Entry RootToDetailAnimation");
}

STATE_END

#pragma mark - StateDetailRootAnimation
STATE(ExpensesNavigationControllerStateChart, StateDetailRootAnimation)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    NSLog(@"Entry DetailRootAnimation");
}

STATE_END

@implementation ExpensesNavigationControllerStateChart {

}

- (id)initWithRootState:(SHPOrState *)rootState {
    _outputPort = [self proxyOutputPortWithProtocol:@protocol(ApplicationStateChartOutputProtocol) name:@"Output"];

    self = [super initWithRootState:rootState];
    if (self) {
    }

    return self;
}


- (void)configureWithConfiguration:(SHPStateChartConfiguration *)configuration {
    StateOverall *stateOverall = [StateOverall new];
    StateExpensesList *stateExpensesList = [StateExpensesList new];
    StateExpenseDetail *stateExpenseDetail = [StateExpenseDetail new];
    StateNavigationBar *stateNavigationBar = [StateNavigationBar new];
    StateRootViewController *stateRootViewController = [StateRootViewController new];
    StateDetailViewController *stateDetailViewController = [StateDetailViewController new];
    StateDetailRootAnimation *stateRootDetailAnimation = [StateDetailRootAnimation new];

    _expensesListStateChart = [[ExpensesListStateChart alloc] initWithRootState:stateExpensesList];
    _expenseDetailStateChart = [[ExpenseDetailStateChart alloc] initWithRootState:stateExpenseDetail];
    _outputPort.proxyChannel = _expensesListStateChart.outputPort.channel = _expenseDetailStateChart.outputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];

    [stateOverall setRegions:@[stateExpensesList, stateExpenseDetail, stateNavigationBar]];
    [stateNavigationBar setSubStates:@[stateRootViewController, stateDetailViewController, stateRootDetailAnimation] initialState:stateRootViewController];

    [stateRootViewController onEvent:@selector(detailPressedForExpense:) transitionTo:stateRootDetailAnimation];
    [stateRootViewController onEvent:@selector(willShowDetailViewController) transitionTo:stateRootDetailAnimation];

    [stateRootDetailAnimation onEvent:@selector(didShowDetailViewController) transitionTo:stateDetailViewController];
    [stateRootDetailAnimation onEvent:@selector(didShowRootViewController) transitionTo:stateRootViewController];

    [stateDetailViewController onEvent:@selector(willShowRootViewController) transitionTo:stateRootDetailAnimation];
    [stateDetailViewController internalTransitionOnEvent:@selector(logout)];

    [configuration.rootState setSubStates:@[stateOverall] initialState:stateOverall];
    [configuration setEventHandlerProtocol:@protocol(ExpensesNavigationControllerStateChartEventProtocol)];
}


@end