// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import "LoginStateChart.h"
#import "SHPStateChartGuard.h"
#import "SHPEventMessage.h"
#import "LoginLib.h"


BASE_STATE(LoginStateChart)

#pragma mark - StateNotLoggedIn
STATE(LoginStateChart, StateNotLoggedIn)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.lib showLoggedIn:NO];
}

STATE_END

#pragma mark - StateLoggedIn
STATE(LoginStateChart, StateLoggedIn)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.lib showLoggedIn:YES];
}

- (void)sessionTimeout {}

STATE_END

#pragma mark - StateLoginFlow
AND_STATE(LoginStateChart, StateLoginFlow)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.eventPort emailFieldChangedTo:@"s093263@student.dtu.dk"];
    [self.stateChart.eventPort passwordFieldChangedTo:@"12345"];
}


- (void)emailFieldChangedTo:(NSString *)email; {
    self.stateChart.email = email;
}

- (void)passwordFieldChangedTo:(NSString *)password {
    self.stateChart.password = password;
}

STATE_END

#pragma mark - StateEmailRegion
STATE(LoginStateChart, StateEmailRegion)STATE_END

#pragma mark - StatePasswordRegion
STATE(LoginStateChart, StatePasswordRegion)STATE_END

#pragma mark - StateLoginButtonRegion
STATE(LoginStateChart, StateLoginButtonRegion)STATE_END

#pragma mark - StateEmailInvalid
STATE(LoginStateChart, StateEmailInvalid)STATE_END

#pragma mark - StateEmailValid
STATE(LoginStateChart, StateEmailValid)STATE_END

#pragma mark - StatePasswordInvalid
STATE(LoginStateChart, StatePasswordInvalid)STATE_END

#pragma mark - StatePasswordValid
STATE(LoginStateChart, StatePasswordValid)
STATE_END

#pragma mark - StateLoginButtonDisabled
STATE(LoginStateChart, StateLoginButtonDisabled)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.lib loginButtonEnabled:NO];
}

STATE_END

#pragma mark - StateLoginButtonEnabled
STATE(LoginStateChart, StateLoginButtonEnabled)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.lib loginButtonEnabled:YES];
}

STATE_END

#pragma mark - StateLoggingIn
STATE(LoginStateChart, StateLoggingIn)STATE_END

@implementation LoginStateChart {
}


- (void)configureWithConfiguration:(SHPStateChartConfiguration *)configuration {
    // Create instance of each state
    StateNotLoggedIn *stateNotLoggedIn = [StateNotLoggedIn new];
    StateLoggedIn *stateLoggedIn = [StateLoggedIn new];
    StateLoginFlow *stateLoginFlow = [StateLoginFlow new];
    StateEmailRegion *stateEmailRegion = [StateEmailRegion new];
    StatePasswordRegion *statePasswordRegion = [StatePasswordRegion new];
    StateLoginButtonRegion *stateLoginButtonRegion = [StateLoginButtonRegion new];
    StateEmailInvalid *stateEmailInvalid = [StateEmailInvalid new];
    StateEmailValid *stateEmailValid = [StateEmailValid new];
    StatePasswordInvalid *statePasswordInvalid = [StatePasswordInvalid new];
    StatePasswordValid *statePasswordValid = [StatePasswordValid new];
    StateLoginButtonDisabled *stateLoginButtonDisabled = [StateLoginButtonDisabled new];
    StateLoginButtonEnabled *stateLoginButtonEnabled = [StateLoginButtonEnabled new];
    StateLoggingIn *stateLoggingIn = [StateLoggingIn new];

    // Setup hierarchy
    [configuration.rootState setSubStates:@[stateNotLoggedIn, stateLoggedIn] initialState:stateNotLoggedIn];
    [stateNotLoggedIn setSubStates:@[stateLoginFlow, stateLoggingIn] initialState:stateLoginFlow];
    [stateLoginFlow setRegions:@[stateEmailRegion, statePasswordRegion, stateLoginButtonRegion]];
    [stateEmailRegion setSubStates:@[stateEmailInvalid, stateEmailValid] initialState:stateEmailInvalid];
    [statePasswordRegion setSubStates:@[statePasswordInvalid, statePasswordValid] initialState:statePasswordInvalid];
    [stateLoginButtonRegion setSubStates:@[stateLoginButtonDisabled, stateLoginButtonEnabled] initialState:stateLoginButtonDisabled];

    // General transitions
    [stateLoginButtonEnabled onEvent:@selector(login) transitionTo:stateLoggingIn];
    [stateLoggedIn onEvent:@selector(logout) transitionTo:stateNotLoggedIn];
//    [stateLoggedIn onTimeout:@10 transitionTo:stateNotLoggedIn event:@selector(sessionTimeout) guard:nil historyMode:HistoryModeNone];
    [stateLoginFlow internalTransitionOnEvent:@selector(emailFieldChangedTo:)];
    [stateLoginFlow internalTransitionOnEvent:@selector(passwordFieldChangedTo:)];

    // Email transitions
    [stateEmailInvalid transitionTo:stateEmailValid guard:[[SHPStateChartGuard alloc] initWithCondition:^BOOL{
        BOOL validEmail =  _email && [_email rangeOfString:@"^.+@.+\\..{2,}$" options:NSRegularExpressionSearch].location != NSNotFound;
        return validEmail;
    } guardDescription:@"Valid email"] reverse:YES];

    // Password transitions
    [statePasswordInvalid transitionTo:statePasswordValid guard:[[SHPStateChartGuard alloc] initWithCondition:^BOOL{
        return _password.length > 4;
    } guardDescription:@"Password length > 4"] reverse:YES];

    // Login button transitions
    [stateLoginButtonDisabled transitionTo:stateLoginButtonEnabled guard:[[SHPStateChartGuard alloc] initWithCondition:^BOOL{
        return stateEmailRegion.activeState == stateEmailValid && statePasswordRegion.activeState == statePasswordValid;
    } guardDescription:@"Email and password valid"] reverse:YES];

    // Login guards
    SHPStateChartGuard *validLoginGuard = [[SHPStateChartGuard alloc] initWithCondition:^BOOL{
        return [_password isEqualToString:@"12345"] && [_email isEqualToString:@"s093263@student.dtu.dk"];
    } guardDescription:@"Email and password valid"];
    [stateLoggingIn transitionTo:stateLoggedIn guard:validLoginGuard];
    [stateLoggingIn transitionTo:stateNotLoggedIn guard:nil];

    // Internal event protocol
    [configuration setEventHandlerProtocol:@protocol(LoginStateChartEventProtocol)];
}

@end