// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <SHPStateChart/SHPProxyOutputPort.h>
#import <SHPStateChart/SHPMessageRouter.h>
#import <SHPStateChart/SHPEventMessage.h>
#import "ModalStateChart.h"
#import "AddExpenseStateChart.h"
#import "TabBarLib.h"
#import "ModalLib.h"
#import "SHPProxyOutputPort.h"
#import "ApplicationStateChart.h"
#import "LoginStateChart.h"
#import "SHPStateChartGuard.h"


BASE_STATE(ModalStateChart)

#pragma mark - StateOverall
AND_STATE(ModalStateChart, StateOverall)STATE_END

#pragma mark - StateSelectedView
STATE(ModalStateChart, StateSelectedView)

//- (void)addNewExpenseSelected {
//    [self.stateChart.lib selectViewControllerAtIndex:0];
//}
//
//- (void)cancelAddExpensePressed {
//    [self.stateChart.lib deselectViewControllerAtIndex:0];
//}
//- (void)createPressedWithAmount:(NSUInteger)amount categoryString:(NSString *)categoryString {
//    [self.stateChart.lib deselectViewControllerAtIndex:0];
//}


STATE_END

#pragma mark - StateAddExpenseSelected
STATE(ModalStateChart, StateAddExpenseSelected)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.lib selectViewControllerAtIndex:0];
}

- (void)eventExit:(SHPEventMessage *)eventMessage {
    [self.stateChart.lib deselectViewControllerAtIndex:0];
}


STATE_END

#pragma mark - StateLoginSelected

STATE(ModalStateChart, StateLoginSelected)
- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.lib selectViewControllerAtIndex:1];
}

- (void)eventExit:(SHPEventMessage *)eventMessage {
    [self.stateChart.lib deselectViewControllerAtIndex:1];
}

STATE_END

#pragma mark - StateNoneSelected
STATE(ModalStateChart, StateNoneSelected)STATE_END

#pragma mark - StateAddExpenseRoot
STATE(ModalStateChart, StateAddExpenseRoot)STATE_END

#pragma mark - StateLoginRoot
STATE(ModalStateChart, StateLoginRoot)STATE_END

@implementation ModalStateChart {

}

- (id)initWithRootState:(SHPOrState *)rootState {
    _outputPort = [self proxyOutputPortWithProtocol:@protocol(ApplicationStateChartOutputProtocol) name:@"Output"];

    self = [super initWithRootState:rootState];
    if (self) {
    }

    return self;
}

- (void)configureWithConfiguration:(SHPStateChartConfiguration *)configuration {
    StateOverall *stateOverall = [StateOverall new];
    StateSelectedView *stateSelectedView = [StateSelectedView new];
    StateNoneSelected *stateNoneSelected = [StateNoneSelected new];
    StateAddExpenseSelected *stateAddExpenseSelected = [StateAddExpenseSelected new];
    StateLoginSelected *stateLoginSelected = [StateLoginSelected new];
    StateAddExpenseRoot *stateAddExpenseRoot = [StateAddExpenseRoot new];
    StateLoginRoot *stateLoginRoot = [StateLoginRoot new];

    _addExpenseStateChart = [[AddExpenseStateChart alloc] initWithRootState:stateAddExpenseRoot];
    _loginStateChart = [[LoginStateChart alloc] initWithRootState:stateLoginRoot];
    _outputPort.proxyChannel = _addExpenseStateChart.outputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];

    [stateOverall setRegions:@[stateSelectedView, stateAddExpenseRoot, stateLoginRoot]];
    [stateSelectedView setSubStates:@[stateNoneSelected, stateAddExpenseSelected, stateLoginSelected] initialState:stateLoginSelected];
    [configuration.rootState setSubStates:@[stateOverall] initialState:stateOverall];

    [stateSelectedView onEvent:@selector(addNewExpenseSelected) transitionTo:stateAddExpenseSelected];
    [stateSelectedView onEvent:@selector(cancelAddExpensePressed) transitionTo:stateNoneSelected];
    [stateSelectedView onEvent:@selector(createPressedWithAmount:categoryString:) transitionTo:stateNoneSelected];

    SHPOrState *loggedIn = (SHPOrState *) [_loginStateChart findStateOfClass:NSClassFromString(@"LoginStateChartStateLoggedIn")];
    [stateLoginSelected transitionTo:stateNoneSelected guard:[[SHPStateChartGuard alloc] initWithCondition:^BOOL{
        return _loginStateChart.rootState.activeState == loggedIn;
    } guardDescription:@"Logged in"] reverse:YES];

    [configuration setEventHandlerProtocol:@protocol(ModalStateChartEventProtocol)];
}

@end