// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <SHPStateChart/SHPStateChartGuard.h>
#import <SHPStateChart/SHPProxyOutputPort.h>
#import <SHPStateChart/SHPMessageRouter.h>
#import "TabBarStateChart.h"
#import "TabBarLib.h"
#import "ModalStateChart.h"
#import "ApplicationStateChart.h"

@protocol TabBarStateChartEventProtocol
@end

BASE_STATE(TabBarStateChart)


#pragma mark - StateTabBar
STATE(TabBarStateChart, StateTabBar)STATE_END

#pragma mark - StateTabExpenses
STATE(TabBarStateChart, StateTabExpenses)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.lib tabAtIndexSelected:0];
    NSLog(@"Entry tab expenses");
}

- (void)expensesTabSelected {
    [self.stateChart.lib popToRootForTabAtIndex:0];
}

STATE_END

#pragma mark - StateTabDebts
STATE(TabBarStateChart, StateTabDebts)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.lib tabAtIndexSelected:1];
    NSLog(@"Entry tab debts");
}

- (void)debtsTabSelected {
    [self.stateChart.lib popToRootForTabAtIndex:1];
}
STATE_END

#pragma mark - StateOverall
AND_STATE(TabBarStateChart, StateOverall)STATE_END

#pragma mark - StateModalView
STATE(ApplicationStateChart, StateModalView)STATE_END

@implementation TabBarStateChart

- (id)initWithRootState:(SHPOrState *)rootState {
    _outputPort = [self proxyOutputPortWithProtocol:@protocol(ApplicationStateChartOutputProtocol) name:@"Output"];

    self = [super initWithRootState:rootState];
    if (self) {
    }

    return self;
}


- (void)configureWithConfiguration:(SHPStateChartConfiguration *)configuration {
    StateOverall *stateOverall = [StateOverall new];
    StateTabBar *stateTabBar = [StateTabBar new];
    StateTabExpenses *stateTabExpenses = [StateTabExpenses new];
    StateTabDebts *stateTabDebts = [StateTabDebts new];
    StateModalView *stateModalView = [StateModalView new];

    _modalStateChart = [[ModalStateChart alloc] initWithRootState:stateModalView];
    _outputPort.proxyChannel = _modalStateChart.outputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];

    [stateOverall setRegions:@[stateTabBar, stateModalView]];

    [stateTabBar setSubStates:@[stateTabExpenses, stateTabDebts] initialState:stateTabExpenses];
    [configuration.rootState setSubStates:@[stateOverall] initialState:stateOverall];

    [stateTabBar onEvent:@selector(expensesTabSelected) transitionTo:stateTabExpenses];
    [stateTabBar onEvent:@selector(debtsTabSelected) transitionTo:stateTabDebts];
}

@end