// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import "AddExpenseLib.h"
#import "AddExpenseViewController.h"
#import "ApplicationStateChart.h"
#import "SHPEventPort.h"
#import "Expense.h"


@implementation AddExpenseLib {

}

- (id)initWithAddExpenseViewController:(AddExpenseViewController *)addExpenseViewController {
    self = [super init];
    if (self) {
        _addExpenseViewController = addExpenseViewController;

        [addExpenseViewController.cancelButton setTarget:self];
        [addExpenseViewController.cancelButton setAction:@selector(cancelPressed)];

        [addExpenseViewController.addButton setTarget:self];
        [addExpenseViewController.addButton setAction:@selector(addPressed)];

        [addExpenseViewController.categoryTextField addTarget:self action:@selector(categoryTextFieldDidChange) forControlEvents:UIControlEventEditingChanged];
        [addExpenseViewController.amountTextField addTarget:self action:@selector(amountTextFieldDidChange) forControlEvents:UIControlEventEditingChanged];
    }

    return self;
}

- (void)amountTextFieldDidChange {
    [self.eventPort amountTextFieldChangedToAmount:self.addExpenseViewController.amountTextField.text];
}

- (void)categoryTextFieldDidChange {
    [self.eventPort categoryTextFieldChangedToCategory:self.addExpenseViewController.categoryTextField.text];
}

- (void)clearData {
    _addExpenseViewController.categoryTextField.text = @"";
    _addExpenseViewController.amountTextField.text = @"";
    [self.eventPort amountTextFieldChangedToAmount:self.addExpenseViewController.amountTextField.text];
    [self.eventPort categoryTextFieldChangedToCategory:self.addExpenseViewController.categoryTextField.text];
}

- (void)addPressed {
    NSInteger amount = [_addExpenseViewController.amountTextField.text intValue];
    NSString *category = [_addExpenseViewController.categoryTextField text];
    [self.eventPort createPressedWithAmount:amount categoryString:category];
}

- (void)cancelPressed {
    [self.eventPort cancelAddExpensePressed];
}

- (void)setAddButtonEnabled:(BOOL)enabled {
    _addExpenseViewController.addButton.enabled = enabled;
}
@end