// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import "DebtsLib.h"
#import "DebtsViewController.h"
#import "ApplicationStateChart.h"
#import "SHPEventPort.h"
#import "Debt.h"
#import "User.h"


@implementation DebtsLib {
    NSArray *_debts;
}

- (id)initWithDebtsViewController:(DebtsViewController *)debtsViewController {
    self = [super init];
    if (self) {
        _debtsViewController = debtsViewController;
        [debtsViewController.refreshButton setTarget:self];
        [debtsViewController.refreshButton setAction:@selector(refreshPressed)];
        [debtsViewController.logoutButton setTarget:self];
        [debtsViewController.logoutButton setAction:@selector(logoutPressed)];

        [debtsViewController.tableView setDataSource:self];
    }

    return self;
}

- (void)logoutPressed {
    [self.eventPort logout];
}

- (void)refreshPressed {
    [self.eventPort refreshDebts];
}

- (void)showDebts:(NSArray *)debts {
    _debts = debts;
    [_debtsViewController.tableView reloadData];
}

#pragma mark - TableView delegate

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return _debts.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"CellIdentifier"];

    if (!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:@"CellIdentifier"];
    }

    Debt *debt= _debts[(NSUInteger) indexPath.row];

    cell.textLabel.text = [NSString stringWithFormat:@"%@ -> %@",debt.giver.name, debt.taker.name];
    cell.detailTextLabel.text = [NSString stringWithFormat:@"%@ kr", debt.amount];

    return cell;
}


@end