// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

static NSString *const ExpensesCellIdentifier = @"ExpensesCellIdentifier";

#import "ExpensesListLib.h"
#import "ExpensesViewController.h"
#import "Expense.h"
#import "User.h"

@implementation ExpensesListLib {
    NSArray *_expenses;
    UITableView *_tableView;
}

- (instancetype)initWithExpensesViewController:(ExpensesViewController *)expensesViewController {
    self = [super init];
    if (self) {
        _expensesViewController = expensesViewController;
        _tableView = expensesViewController.tableView;
        _tableView.delegate = self;
        _tableView.dataSource = self;

        [expensesViewController view];
        [expensesViewController.refreshButton setTarget:self];
        [expensesViewController.refreshButton setAction:@selector(didSelectRefresh)];

        [expensesViewController.addButton setTarget:self];
        [expensesViewController.addButton setAction:@selector(didSelectAddExpense)];
    }

    return self;
}

- (void)didSelectAddExpense {
    [self.eventPort addNewExpenseSelected];
}

- (void)didSelectRefresh {
    [self.eventPort refreshExpenses];
}

- (void)showExpenses:(NSArray *)expenses {
    _expenses = expenses;

    [_tableView reloadData];
}

#pragma mark - TableView delegate

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return _expenses.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:ExpensesCellIdentifier];

    if (!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:ExpensesCellIdentifier];
        [cell setAccessoryType:UITableViewCellAccessoryDisclosureIndicator];
    }

    Expense *expense = _expenses[(NSUInteger) indexPath.row];

    cell.textLabel.text = expense.payer.name;
    cell.detailTextLabel.text = [NSString stringWithFormat:@"%@ kr",expense.amount];

    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    Expense *expense = _expenses[(NSUInteger) indexPath.row];
    [self.eventPort detailPressedForExpense:expense];
}

@end