// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <UIKit/UIKit.h>
#import "ExpensesNavigationBarLib.h"
#import "ExpensesViewController.h"
#import "ExpensesListLib.h"
#import "ExpenseDetailViewController.h"

@implementation ExpensesNavigationBarLib {

}

- (instancetype)initWithNavigationController:(UINavigationController *)navigationController {
    self = [super init];
    if (self) {
        _navigationController = navigationController;
        [_navigationController setDelegate:self];
    }

    return self;
}

- (void)transitionToViewControllerAtIndex:(NSUInteger)index {
    [self.navigationController pushViewController:_viewControllers[index] animated:YES];
}

- (void)popToRoot {
    [self.navigationController popToRootViewControllerAnimated:YES];
}

#pragma mark - NavigationController delegate

- (void)navigationController:(UINavigationController *)navigationController willShowViewController:(UIViewController *)viewController animated:(BOOL)animated {
    NSUInteger index = [self.navigationController.viewControllers indexOfObject:viewController];

    if(index == 0) {
        [self.eventPort willShowRootViewController];
    } else if (index == 1) {
        [self.eventPort willShowDetailViewController];

    }

    [[self.navigationController transitionCoordinator] notifyWhenInteractionEndsUsingBlock:^(id<UIViewControllerTransitionCoordinatorContext> context) {
                if([context isCancelled]) {
                    [self.eventPort willShowDetailViewController];
                    NSTimeInterval animationCompletion = [context transitionDuration] * [context percentComplete];
                    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (uint64_t)animationCompletion * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
                        [self.eventPort didShowDetailViewController];
                    });
                }
            }];

}

- (void)navigationController:(UINavigationController *)navigationController didShowViewController:(UIViewController *)viewController animated:(BOOL)animated {
    NSUInteger index = [self.navigationController.viewControllers indexOfObject:viewController];

    if(index == 0) {
        [self.eventPort didShowRootViewController];
    } else if (index == 1) {
        [self.eventPort didShowDetailViewController];

    }
}

@end