// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <UIKit/UIKit.h>
#import "TabBarLib.h"
#import "SHPEventPort.h"
#import "ApplicationStateChart.h"
#import "ExpensesNavigationBarLib.h"
#import "DebtsViewController.h"


@implementation TabBarLib {

}

- (instancetype)initWithTabBarController:(UITabBarController *)tabBarController {
    self = [super init];
    if (self) {
        _tabBarController = tabBarController;
        [_tabBarController setDelegate:self];
    }

    return self;
}

- (void)setEventPort:(SHPEventPort <ApplicationStateChartEventProtocol> *)eventPort {
    _eventPort = eventPort;

    self.expensesNavigationBarLib.eventPort = eventPort;
}


- (void)tabAtIndexSelected:(NSUInteger)index {
    [_tabBarController setSelectedIndex:index];
}

- (void)popToRootForTabAtIndex:(NSUInteger)index {
    UIViewController *viewController = _tabBarController.viewControllers[index];
    if ([viewController isKindOfClass:[UINavigationController class]]) {
        UINavigationController *navigationController = (UINavigationController *) viewController;
        [navigationController popToRootViewControllerAnimated:YES];
    }
}

#pragma mark - TabBarController delegate

- (BOOL)tabBarController:(UITabBarController *)tabBarController shouldSelectViewController:(UIViewController *)viewController {
    NSUInteger index = [tabBarController.viewControllers indexOfObject:viewController];

    if (index == 0) {
        [self.eventPort expensesTabSelected];
    } else if (index == 1) {
        [self.eventPort debtsTabSelected];
    }

    return NO;
}


@end