// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <Foundation/Foundation.h>
#import "SHPComponent.h"
#import "SHPOrState.h"
#import "SHPAndState.h"
#import "SHPEventPort.h"

#define BASE_STATE(prefix) @interface prefix##OrState : SHPOrState <prefix##EventProtocol> - (prefix *)stateChart; @end @compatibility_alias OrState prefix##OrState; @implementation prefix##OrState - (prefix *)stateChart { return (prefix *) super.stateChart; } @end @interface prefix##AndState : SHPAndState <prefix##EventProtocol> - (prefix *)stateChart; @end @compatibility_alias AndState prefix##AndState; @implementation prefix##AndState - (prefix *)stateChart { return (prefix *) super.stateChart; } @end

#define STATE(prefix,stateName) @interface prefix##stateName : OrState @end @compatibility_alias stateName prefix##stateName; @implementation prefix##stateName - (NSString *)name { return (__bridge NSString *) CFSTR(#stateName); }
#define AND_STATE(prefix,stateName) @interface prefix##stateName : AndState @end @compatibility_alias stateName prefix##stateName; @implementation prefix##stateName - (NSString *)name { return (__bridge NSString *) CFSTR(#stateName); }

#define DECLARE_BASE_STATE(prefix) @interface prefix##OrState : SHPOrState <prefix##EventProtocol> - (prefix *)stateChart; @end @interface prefix##AndState : SHPAndState <prefix##EventProtocol> - (prefix *)stateChart; @end
#define DECLARE_STATE(prefix,stateName) @interface prefix##stateName : prefix##OrState @end
#define DECLARE_AND_STATE(prefix,stateName) @interface prefix##stateName : prefix##AndState @end

#define IMPLEMENT_BASE_STATE(prefix) @implementation prefix##OrState - (prefix *)stateChart { return (prefix *) super.stateChart; } @end @implementation prefix##AndState - (prefix *)stateChart { return (prefix *) super.stateChart; } @end @compatibility_alias OrState prefix##OrState; @compatibility_alias AndState prefix##AndState;
#define IMPLEMENT_STATE(prefix,stateName) @compatibility_alias stateName prefix##stateName; @implementation prefix##stateName - (NSString *)name { return (__bridge NSString *) CFSTR(#stateName); }
#define IMPLEMENT_AND_STATE(prefix,stateName) @compatibility_alias stateName prefix##stateName; @implementation prefix##stateName - (NSString *)name { return (__bridge NSString *) CFSTR(#stateName); }

#define STATE_END @end

@class SHPInputPort;
@class SHPOutputPort;
@class SHPEventMessage;
@class SHPStateChartConfiguration;
@class SHPOrState;
@class SHPEventHandlingResult;
@class SHPStateRepresentation;
@class SHPStateChartRepresentation;

@protocol SHPStateChartConfigurationProtocol <NSObject>

@property (nonatomic) Protocol *eventHandlerProtocol;
@property (nonatomic, readonly) SHPOrState *rootState;

@end

@protocol SHPStateChartConfigureProtocol <NSObject>
@optional
- (void)configureWithConfiguration:(SHPStateChartConfiguration *)configuration;
@end

@interface SHPStateChartConfiguration : NSObject <SHPStateChartConfigurationProtocol>

@property (nonatomic) Protocol *eventHandlerProtocol;
@property (nonatomic, readonly) SHPOrState *rootState;

- (id)initWithRootState:(SHPOrState *)rootState;
@end

@protocol SHPStateChartInfoProtocol
@optional
- (void)didAddInitialEventHandlingResult:(SHPEventHandlingResult *)eventHandlingResult;
- (void)didAddEventHandlingResult:(SHPEventHandlingResult *)eventHandlingResult;
- (void)didUpdateStateChartRepresentation:(SHPStateChartRepresentation *)stateChartRepresentation;
@end

@interface SHPStateChart : SHPComponent <SHPStateChartConfigureProtocol>

@property (nonatomic, readonly) SHPOrState *rootState;
@property (nonatomic, readonly) SHPEventPort *eventPort;
@property (nonatomic, readonly) SHPOutputPort<SHPStateChartInfoProtocol> *stateChartInfoPort;

- (id)initWithRootState:(SHPOrState *)rootState;
- (void)initialTransition;
- (BOOL)performEvent:(SHPEventMessage *)eventMessage;
- (SHPState *)findStateOfClass:(Class)stateClass;

@end
