// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <Foundation/Foundation.h>

@interface NSObject(SHPDelegation)

/// ---------------------------------------------------------------------
/// @name Delegation
/// ---------------------------------------------------------------------

/**
 Perform a delegation method only if there is a delegate and if it responds to the method

 Provides a single call which checks if a delegate is set and whether it responds to the
 selector before trying to call it.
 
 @warning *Warning*
 
 It is assumed that the delegate can be accessed through a property called delegate
*/
- (void)performDelegateSelector:(SEL)selector withObject:(id)object;


/**
 Same as - (void)performDelegateSelector:(SEL)selector withObject:(id)object, but takes
 an additional object parameter to be sent to the delegate
*/
- (void)performDelegateSelector:(SEL)selector withObject:(id)object1 withObject:(id)object2;

@end
