// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <Foundation/Foundation.h>
#import "SHPChannel.h"
#import "SHPInputPort.h"
#import "SHPOutputPort.h"

@class SHPEventMessage;
@class SHPComponentRegister;
@class SHPInputPort;
@class SHPOutputPort;
@protocol SHPComponentRegisterCommandProtocol;

@interface SHPComponent : NSObject <SHPInputPortProtocol, SHPOutputPortProtocol> {
    @protected
    dispatch_queue_t _messageQueue;
}

@property (nonatomic, readonly) dispatch_queue_t messageQueue;
@property (nonatomic, readonly) NSString *name;
@property (nonatomic, readonly) NSString *identifier;
@property (nonatomic, readonly) NSArray *ports;
@property (nonatomic, readonly) NSArray *subComponents;
@property (nonatomic) SHPComponent *superComponent;
@property (nonatomic, readonly) NSMutableDictionary *portTraceDictionary;
@property (nonatomic, readonly) SHPOutputPort<SHPComponentRegisterCommandProtocol> *componentRegisterPort;

- (id)initWithComponentRegisterChannel:(SHPChannel *)componentRegisterChannel;
- (id)initWithComponentRegisterChannel:(SHPChannel *)componentRegisterChannel name:(NSString *)name;

- (dispatch_queue_t)defaultMessageQueue;
- (NSString *)messageQueueName;

- (id)inputPortWithProtocol:(Protocol *)protocol name:(NSString *)name;
- (id)outputPortWithProtocol:(Protocol *)protocol name:(NSString *)name;
- (id)proxyInputPortWithProtocol:(Protocol *)protocol name:(NSString *)name;
- (id)proxyOutputPortWithProtocol:(Protocol *)protocol name:(NSString *)name;
- (void)addSubComponent:(SHPComponent *)subComponent;
@end
