// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <Foundation/Foundation.h>
#import "SHPStateChart.h"
#import "SHPInputPort.h"
#import "SHPOutputPort.h"

@class GCDAsyncSocket;
@protocol SHPSocketClientCommandProtocol;
@protocol SHPSocketClientNotificationProtocol;

@protocol SHPSocketServerCommandProtocol
@optional
- (void)eventSocketServerStart;
- (void)eventSocketServerStop;
- (void)eventSocketServerSendObject:(id <NSCoding>)object;
@end

@protocol SHPSocketServerNotificationProtocol
@optional
- (void)eventSocketServerReceivedObject:(id)object;
- (void)eventSocketServerStopped;
- (void)eventSocketServerFailedWithError:(NSError *)error;
- (void)eventSocketServerStarted;
@end

@interface SHPSocketServerStateChart : SHPStateChart

@property(nonatomic, readonly) GCDAsyncSocket *serverSocket;
@property(nonatomic, readonly) uint16_t port;
@property(nonatomic, readonly) NSString *interface;

@property (nonatomic, readonly) SHPInputPort<SHPSocketServerCommandProtocol> *commandInputPort;
@property (nonatomic, readonly) SHPOutputPort<SHPSocketServerNotificationProtocol> *notificationOutputPort;

@property (nonatomic, readonly) SHPOutputPort<SHPSocketClientCommandProtocol> *commandOutputPort;
@property (nonatomic, readonly) SHPInputPort<SHPSocketClientNotificationProtocol> *notificationInputPort;

- (instancetype)initWithPort:(uint16_t)port interface:(NSString *)interface;

@end