// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <XCTest/XCTest.h>
#import "SHPStateChart.h"
#import "Scenario1StateChart.h"
#import "StateChartTestBase.h"
#import "SHPTestPort.h"


@interface Scenario1Tests : StateChartTestBase

@property (nonatomic, readonly) SHPTestPort<Scenario1StateChartEventProtocol> *port;

@end

@implementation Scenario1Tests {
}


- (void)setUp
{
    [super setUp];

    [self setUpWithPortProtocol:@protocol(Scenario1StateChartEventProtocol) stateChartClass:[Scenario1StateChart class]];
}

- (void)tearDown
{
    [super tearDown];
}

- (void)testEvent1 {
    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event1];
    } eventSelector:@selector(event1)];

    NSArray *sequence = @[EXIT(@"State2"), ACTION(@"State2"), ENTRY(@"State1"), ENTRY(@"State1a"), ENTRY(@"State1a1"), ENTRY(@"State1b"), ENTRY(@"State1b1")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

- (void)testEvent1Event2 {
    [self.port stateRecordingsForEventBlock:^{
        [self.port event1];
    } eventSelector:@selector(event1)];

    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event2];
    } eventSelector:@selector(event2)];

    NSArray *sequence = @[EXIT(@"State1a1"), EXIT(@"State1a"), EXIT(@"State1b1"), EXIT(@"State1b"), EXIT(@"State1"), ACTION(@"State1b1"), ENTRY(@"State2")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

- (void)testEven1Event3 {
    [self.port stateRecordingsForEventBlock:^{
        [self.port event1];
    } eventSelector:@selector(event1)];

    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event3];
    } eventSelector:@selector(event3)];

    NSArray *sequence = @[EXIT(@"State1a1"), EXIT(@"State1a"), EXIT(@"State1b1"), EXIT(@"State1b"), EXIT(@"State1"), ACTION(@"State1"), ENTRY(@"State2")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}


@end

