// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <XCTest/XCTest.h>
#import "StateChartTestBase.h"
#import "Scenario6StateChart.h"
#import "SHPTestPort.h"

@interface Scenario6Tests : StateChartTestBase
@property (nonatomic, readonly) SHPTestPort<Scenario6StateChartEventProtocol> *port;
@end

@implementation Scenario6Tests

- (void)setUp
{
    [super setUp];

    [self setUpWithPortProtocol:@protocol(Scenario6StateChartEventProtocol) stateChartClass:[Scenario6StateChart class]];
}

- (void)tearDown
{
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

- (void)testEvent1 {
    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event1];
    } eventSelector:@selector(event1)];

    NSArray *sequence = @[EXIT(@"State1a1"), ACTION(@"State1a1"), ENTRY(@"State1a2"), ACTION(@"State1b1")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

- (void)testEvent3 {
    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event3];
    } eventSelector:@selector(event3)];

    NSArray *sequence = @[EXIT(@"State1a1"), EXIT(@"State1a"), EXIT(@"State1b1"), EXIT(@"State1b"), EXIT(@"State1"), ACTION(@"State1"), ENTRY(@"State2"), ENTRY(@"State2a"), ENTRY(@"State2a1")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

- (void)testEvent3Event1 {
    [self.port stateRecordingsForEventBlock:^{
        [self.port event3];
    } eventSelector:@selector(event3)];

    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event1];
    } eventSelector:@selector(event1)];

    NSArray *sequence = @[EXIT(@"State2a1"), EXIT(@"State2a"), ACTION(@"State2"), ENTRY(@"State2a"), ENTRY(@"State2a1")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

- (void)testEvent3Event2 {
    [self.port stateRecordingsForEventBlock:^{
        [self.port event3];
    } eventSelector:@selector(event3)];

    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event2];
    } eventSelector:@selector(event2)];

    NSArray *sequence = @[EXIT(@"State2a1"), ACTION(@"State2a1"), ENTRY(@"State2a1")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}


@end
