// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <XCTest/XCTest.h>
#import "Scenario7StateChart.h"
#import "SHPTestPort.h"
#import "StateChartTestBase.h"

@interface Scenario7Tests : StateChartTestBase
@property (nonatomic, readonly) SHPTestPort<Scenario7StateChartEventProtocol> *port;

@end

@implementation Scenario7Tests

- (void)setUp
{
    [super setUp];

    [self setUpWithPortProtocol:@protocol(Scenario7StateChartEventProtocol) stateChartClass:[Scenario7StateChart class]];
}

- (void)tearDown
{
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

- (void)testEvent1 {
    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event1];
    } eventSelector:@selector(event1)];

    NSArray *sequence = @[EXIT(@"StateA1"), ACTION(@"StateA1"), ENTRY(@"StateA2")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

- (void)testEvent2 {
    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event2];
    } eventSelector:@selector(event2)];

    NSArray *sequence = @[EXIT(@"StateB1"), ACTION(@"StateB1"), ENTRY(@"StateB2")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

- (void)testEvent3 {
    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event3];
    } eventSelector:@selector(event3)];

    NSArray *sequence = @[EXIT(@"StateC1"), ACTION(@"StateC1"), ENTRY(@"StateC2")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

- (void)testEvent1Event2Event3 {
    NSMutableArray *recordings = [NSMutableArray new];

    [recordings addObjectsFromArray:[self.port stateRecordingsForEventBlock:^{ [self.port event1]; }  eventSelector:@selector(event1)]];
    [recordings addObjectsFromArray:[self.port stateRecordingsForEventBlock:^{ [self.port event2]; } eventSelector:@selector(event2)]];
    [recordings addObjectsFromArray:[self.port stateRecordingsForEventBlock:^{ [self.port event3]; } eventSelector:@selector(event3)]];

    NSArray *sequence = @[EXIT(@"StateA1"), ACTION(@"StateA1"), ENTRY(@"StateA2"),
            EXIT(@"StateB1"), ACTION(@"StateB1"), ENTRY(@"StateB2"),
            EXIT(@"StateC1"), ACTION(@"StateC1"), ENTRY(@"StateC2"),
            EXIT(@"StateD1"), ACTION(@"StateD1"), ENTRY(@"StateD2")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

@end
