// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <XCTest/XCTest.h>
#import "StateChartTestBase.h"
#import "Scenario8StateChart.h"
#import "SHPTestPort.h"

@interface Scenario8Tests : StateChartTestBase
@property (nonatomic, readonly) SHPTestPort<Scenario8StateChartEventProtocol> *port;
@end

@implementation Scenario8Tests

- (void)setUp
{
    [super setUp];

    [self setUpWithPortProtocol:@protocol(Scenario8StateChartEventProtocol) stateChartClass:[Scenario8StateChart class]];
}

- (void)tearDown
{
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

- (void)testEvent1 {
    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event1];
    } eventSelector:@selector(event1)];

    NSArray *sequence = @[EXIT(@"State1"), ACTION(@"State1"), ENTRY(@"State2"), ENTRY(@"State2a"), ENTRY(@"State2a1")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

- (void)testEvent1Event2 {
    [self.port stateRecordingsForEventBlock:^{
        [self.port event1];
    } eventSelector:@selector(event1)];

    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event2];
    } eventSelector:@selector(event2)];

    NSArray *sequence = @[EXIT(@"State2a1"), EXIT(@"State2a"), ACTION(@"State2a"), ENTRY(@"State2b"), ENTRY(@"State2b1")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

- (void)testEvent1Event2Event1 {
    [self.port stateRecordingsForEventBlock:^{ [self.port event1]; } eventSelector:@selector(event1)];
    [self.port stateRecordingsForEventBlock:^{ [self.port event2]; } eventSelector:@selector(event2)];

    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event1];
    } eventSelector:@selector(event1)];

    NSArray *sequence = @[EXIT(@"State2b1"), EXIT(@"State2b"), EXIT(@"State2"), ACTION(@"State2"), ENTRY(@"State1")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

- (void)testEvent1Event2Event1Event1 {
    [self.port stateRecordingsForEventBlock:^{ [self.port event1]; } eventSelector:@selector(event1)];
    [self.port stateRecordingsForEventBlock:^{ [self.port event2]; } eventSelector:@selector(event2)];
    [self.port stateRecordingsForEventBlock:^{ [self.port event1]; } eventSelector:@selector(event1)];

    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event1];
    } eventSelector:@selector(event1)];

    NSArray *sequence = @[EXIT(@"State1"), ACTION(@"State1"), ENTRY(@"State2"), ENTRY(@"State2b"), ENTRY(@"State2b1")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

@end
