// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <XCTest/XCTest.h>
#import "Scenario9StateChart.h"
#import "StateChartTestBase.h"
#import "SHPTestPort.h"

@interface Scenario9Tests : StateChartTestBase
@property (nonatomic, readonly) SHPTestPort<Scenario9StateChartEventProtocol> *port;
@end

@implementation Scenario9Tests

- (void)setUp
{
    [super setUp];

    [self setUpWithPortProtocol:@protocol(Scenario9StateChartEventProtocol) stateChartClass:[Scenario9StateChart class]];
}

- (void)tearDown
{
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

- (void)testEvent2 {
    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event2];
    } eventSelector:@selector(event2)];

    NSArray *sequence = @[EXIT(@"State1"), ACTION(@"State1"), ENTRY(@"Submachines"), ENTRY(@"StateRegion1"), ENTRY_WITH_PREFIX(@"StateSM1", @"Submachine1StateChart"),
            ENTRY_WITH_PREFIX(@"StateSM1a", @"Submachine1StateChart"), ENTRY(@"StateRegion2"),ENTRY_WITH_PREFIX(@"StateSM1", @"Submachine1StateChart"),
            ENTRY_WITH_PREFIX(@"StateSM1a", @"Submachine1StateChart"), ENTRY(@"StateRegion3"), ENTRY_WITH_PREFIX(@"StateSM1", @"Submachine1StateChart"),
            ENTRY_WITH_PREFIX(@"StateSM1a", @"Submachine1StateChart")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

- (void)testEvent2Event1 {
    [self.port stateRecordingsForEventBlock:^{ [self.port event2]; } eventSelector:@selector(event2)];

    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event1];
    } eventSelector:@selector(event1)];

    NSArray *sequence = @[EXIT_WITH_PREFIX(@"StateSM1a", @"Submachine1StateChart"), ACTION_WITH_PREFIX(@"StateSM1a", @"Submachine1StateChart"), ENTRY_WITH_PREFIX(@"StateSM1b", @"Submachine1StateChart"),
            EXIT_WITH_PREFIX(@"StateSM1a", @"Submachine1StateChart"), ACTION_WITH_PREFIX(@"StateSM1a", @"Submachine1StateChart"),ENTRY_WITH_PREFIX(@"StateSM1b", @"Submachine1StateChart"),
            EXIT_WITH_PREFIX(@"StateSM1a", @"Submachine1StateChart"), ACTION_WITH_PREFIX(@"StateSM1a", @"Submachine1StateChart"),ENTRY_WITH_PREFIX(@"StateSM1b", @"Submachine1StateChart")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

- (void)testEvent2Event1Event3 {
    [self.port stateRecordingsForEventBlock:^{ [self.port event2]; } eventSelector:@selector(event2)];
    [self.port stateRecordingsForEventBlock:^{ [self.port event1]; } eventSelector:@selector(event1)];

    NSArray *recordings = [self.port stateRecordingsForEventBlock:^{
        [self.port event3];
    } eventSelector:@selector(event3)];

    NSArray *sequence = @[EXIT_WITH_PREFIX(@"StateSM1b", @"Submachine1StateChart"), EXIT_WITH_PREFIX(@"StateSM1", @"Submachine1StateChart"), ACTION_WITH_PREFIX(@"StateSM1b", @"Submachine1StateChart"), ENTRY(@"SubmachineState1"),
            EXIT_WITH_PREFIX(@"StateSM1b", @"Submachine1StateChart"), EXIT_WITH_PREFIX(@"StateSM1", @"Submachine1StateChart"), ACTION_WITH_PREFIX(@"StateSM1b", @"Submachine1StateChart"), ENTRY(@"SubmachineState2"),
            EXIT_WITH_PREFIX(@"StateSM1b", @"Submachine1StateChart"), EXIT_WITH_PREFIX(@"StateSM1", @"Submachine1StateChart"), ACTION_WITH_PREFIX(@"StateSM1b", @"Submachine1StateChart"), ENTRY(@"SubmachineState3")];

    [self validateValidationSequence:sequence forRecordings:recordings];
}

@end
