// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <Foundation/Foundation.h>
#import <XCTest/XCTest.h>
#import "SHPEventRepresentation.h"

@class SHPTestPort;
@class SHPInputPort;
@class SHPStateChartTestValidation;

#define EXIT(stateClassName) [self exit:stateClassName prefix:self.stateChartName]
#define EXIT_WITH_PREFIX(stateClassName, pre) [self exit:stateClassName prefix:pre]
#define ENTRY(stateClassName) [self entry:stateClassName prefix:self.stateChartName]
#define ENTRY_WITH_PREFIX(stateClassName, pre) [self entry:stateClassName prefix:pre]
#define ACTION(stateClassName) [self action:stateClassName prefix:self.stateChartName]
#define ACTION_WITH_PREFIX(stateClassName, pre) [self action:stateClassName prefix:pre]
#define INITIAL(stateClassName) [self initial:stateClassName prefix:self.stateChartName]
#define INITIAL_WITH_PREFIX(stateClassName, pre) [self initial:stateClassName prefix:pre]


@protocol StateChartTestBaseProtocol
@property (nonatomic, readonly) SHPInputPort *eventInPort;
@end

typedef NS_ENUM(NSInteger, SHPStateChartTestEventType) {
    SHPStateChartTestEventTypeEntry,
    SHPStateChartTestEventTypeExit,
    SHPStateChartTestEventTypeInitial,
    SHPStateChartTestEventTypeAction
};

@interface SHPStateChartTestValidation : NSObject
@property (nonatomic, strong) NSString *stateClass;
@property (nonatomic) SHPEventRepresentationEventType eventType;
@end

@interface StateChartTestBase : XCTestCase

@property (nonatomic, readonly) SHPTestPort *port;
@property (nonatomic, readonly) NSString *stateChartName;

- (void)setUpWithPortProtocol:(Protocol *)portProtocol stateChartClass:(Class)stateChartClass;
- (void)validateValidationSequence:(NSArray *)validationSequence forRecordings:(NSArray *)recordings;

- (SHPStateChartTestValidation *)exit:(NSString *)stateClassName prefix:(NSString *)prefix;
- (SHPStateChartTestValidation *)entry:(NSString *)stateClassName prefix:(NSString *)prefix;
- (SHPStateChartTestValidation *)initial:(NSString *)stateClassName prefix:(NSString *)prefix;
- (SHPStateChartTestValidation *)action:(NSString *)stateClassName prefix:(NSString *)prefix;
@end