// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <SHPStateChart/SHPInputPort.h>
#import "StateChartTestBase.h"
#import "SHPMessageRouter.h"
#import "SHPStateChart.h"
#import "SHPTestPort.h"
#import "SHPTransitionRepresentation.h"
#import "SHPEventHandlingResult.h"
#import "SHPStateRepresentation.h"


@implementation SHPStateChartTestValidation {
}
@end



@implementation StateChartTestBase {
}

- (void)setUpWithPortProtocol:(Protocol *)portProtocol stateChartClass:(Class)stateChartClass {
    _stateChartName = NSStringFromClass(stateChartClass);

    _port = [[SHPTestPort alloc] initWithProtocol:portProtocol];

    SHPStateChart<StateChartTestBaseProtocol> *stateChart = [stateChartClass new];

    _port.channel = stateChart.eventInPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];
    _port.inputChannel  = stateChart.stateChartInfoPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];

    [stateChart initialTransition];
}

- (void)validateValidationSequence:(NSArray *)validationSequence forRecordings:(NSArray *)recordings {
    SHPTransitionRepresentation *transition = recordings.firstObject;

    __block NSUInteger validationIndex = 0;
    [transition.eventRepresentations enumerateObjectsUsingBlock:^(SHPEventRepresentation *eventRepresentation, NSUInteger idx, BOOL *stop) {
        if (eventRepresentation.eventType == SHPEventRepresentationEventTypeInitial) {} else {
            SHPStateChartTestValidation *validation = [validationSequence objectAtIndex:validationIndex];
            validationIndex++;
            SHPEventRepresentationEventType recordingEventType = eventRepresentation.eventType;//transitionRepresentation.event;//[self eventTypeForSelector:recording.eventType];

            NSString *eventTypeErrorString = [NSString stringWithFormat:@"Event type expected to be %@ but was %@", [self nameForEventType:validation.eventType], [self nameForEventType:recordingEventType]];
            NSAssert(recordingEventType == validation.eventType, eventTypeErrorString);


            NSString *eventClassErrorString = [NSString stringWithFormat:@"Class of state expected to be %@ but was %@", validation.stateClass, eventRepresentation.stateRepresentation.uniqueName];
            NSAssert([eventRepresentation.stateRepresentation.uniqueName isEqualToString:validation.stateClass], eventClassErrorString);
        }
    }];

}

- (SHPStateChartTestEventType)eventTypeForSelector:(SEL)selector {
    if(selector == @selector(eventEntry:)) {
        return SHPStateChartTestEventTypeEntry;
    } else if(selector == @selector(eventExit:)) {
        return SHPStateChartTestEventTypeExit;
    } else if(selector == @selector(eventInitial:)) {
        return SHPStateChartTestEventTypeInitial;
    } else {
        return SHPStateChartTestEventTypeAction;
    }
}

- (NSString *)nameForEventType:(SHPEventRepresentationEventType)eventType {
    switch (eventType) {
        case SHPEventRepresentationEventTypeInitial: return @"EventTypeInitial";
        case SHPEventRepresentationEventTypeExit: return @"EventTypeExit";
        case SHPEventRepresentationEventTypeEntry: return @"EventTypeEntry";
        case SHPEventRepresentationEventTypeAction: return @"EventTypeAction";
    }

    return @"Unknown event type";
}

- (SHPStateChartTestValidation *)exit:(NSString *)stateClassName prefix:(NSString *)prefix {
    SHPStateChartTestValidation *validation = [SHPStateChartTestValidation new];
    validation.eventType = SHPEventRepresentationEventTypeExit;
    validation.stateClass = [NSString stringWithFormat:@"%@%@",prefix,stateClassName];

    return validation;
}

- (SHPStateChartTestValidation *)entry:(NSString *)stateClassName prefix:(NSString *)prefix {
    SHPStateChartTestValidation *validation = [SHPStateChartTestValidation new];
    validation.eventType = SHPEventRepresentationEventTypeEntry;
    validation.stateClass = [NSString stringWithFormat:@"%@%@",prefix,stateClassName];

    return validation;
}

- (SHPStateChartTestValidation *)initial:(NSString *)stateClassName prefix:(NSString *)prefix {
    SHPStateChartTestValidation *validation = [SHPStateChartTestValidation new];
    validation.eventType = SHPEventRepresentationEventTypeInitial;
    validation.stateClass = [NSString stringWithFormat:@"%@%@",prefix,stateClassName];

    return validation;
}

- (SHPStateChartTestValidation *)action:(NSString *)stateClassName prefix:(NSString *)prefix {
    SHPStateChartTestValidation *validation = [SHPStateChartTestValidation new];
    validation.eventType = SHPEventRepresentationEventTypeAction;
    validation.stateClass = [NSString stringWithFormat:@"%@%@",prefix,stateClassName];

    return validation;
}


@end