// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.


#import "Scenario7StateChart.h"
#import "SHPStateChartGuard.h"

BASE_STATE(Scenario7StateChart)

STATE(Scenario7StateChart, StateTransient)

- (void)actionGuard1 {
    NSLog(@"actionGuard1");
}

- (void)actionGuard2 {
    NSLog(@"actionGuard2");
}

- (void)actionTransient {
    NSLog(@"actionTransient");
}

STATE_END

AND_STATE(Scenario7StateChart, StateS)STATE_END
STATE(Scenario7StateChart, StateA)STATE_END
STATE(Scenario7StateChart, StateA1)STATE_END
STATE(Scenario7StateChart, StateA2)STATE_END

STATE(Scenario7StateChart, StateB)STATE_END
STATE(Scenario7StateChart, StateB1)STATE_END
STATE(Scenario7StateChart, StateB2)STATE_END

STATE(Scenario7StateChart, StateC)STATE_END
STATE(Scenario7StateChart, StateC1)STATE_END
STATE(Scenario7StateChart, StateC2)STATE_END

STATE(Scenario7StateChart, StateD)STATE_END
STATE(Scenario7StateChart, StateD1)STATE_END
STATE(Scenario7StateChart, StateD2)STATE_END

STATE(Scenario7StateChart, StateE)STATE_END
STATE(Scenario7StateChart, StateF)STATE_END


@implementation Scenario7StateChart

- (id)init {
    if (!(self = [super init])) {return nil;}

    _eventInPort = [self inputPortWithProtocol:@protocol(Scenario7StateChartEventProtocol) name:@"Event In"];

    return self;
}

- (void)configureWithConfiguration:(id <SHPStateChartConfigurationProtocol>)configuration {
    StateTransient *ST  = [StateTransient new];

    StateS *S = [StateS new];

    StateA *SA      = [StateA new];
    StateA1 *SA1    = [StateA1 new];
    StateA2 *SA2    = [StateA2 new];

    StateB *SB      = [StateB new];
    StateB1 *SB1    = [StateB1 new];
    StateB2 *SB2    = [StateB2 new];

    StateC *SC      = [StateC new];
    StateC1 *SC1    = [StateC1 new];
    StateC2 *SC2    = [StateC2 new];

    StateD *SD      = [StateD new];
    StateD1 *SD1    = [StateD1 new];
    StateD2 *SD2    = [StateD2 new];

    StateE *stateE = [StateE new];
    StateF *stateF = [StateF new];

    [SA setSubStates:@[SA1, SA2] initialState:SA1];
    [SB setSubStates:@[SB1, SB2] initialState:SB1];
    [SC setSubStates:@[SC1, SC2] initialState:SC1];
    [SD setSubStates:@[SD1, SD2] initialState:SD1];

    [S setRegions:@[SA, SB, SC, SD]];
    [configuration.rootState setSubStates:@[ST, S] initialState:ST];

    SHPStateChartGuard *guardST = [SHPStateChartGuard guardWithCondition:^BOOL {
        return YES;
    } guardDescription:@"1==1"];
    SHPStateChartGuard *guardST2 = [SHPStateChartGuard guardWithCondition:^BOOL {
        return NO;
    } guardDescription:@"1==2"];

    [ST transitionTo:S action:@selector(actionGuard1) guard:guardST];
    [ST transitionTo:stateF action:@selector(actionGuard2) guard:guardST2];
    [ST transitionTo:stateE action:@selector(actionTransient)];

    [SA1 onEvent:@selector(event1) transitionTo:SA2];
    [SB1 onEvent:@selector(event2) transitionTo:SB2];
    [SC1 onEvent:@selector(event3) transitionTo:SC2];

    [SA2 onEvent:@selector(event4) transitionTo:SA1];
    [SB2 onEvent:@selector(event4) transitionTo:SB1];
    [SC2 onEvent:@selector(event4) transitionTo:SC1];

    SHPStateChartGuard *guardD1ToD2 = [SHPStateChartGuard guardWithCondition:^BOOL {
        return SA.activeState == SA2 && SB.activeState == SB2 && SC.activeState == SC2;
    } guardDescription:@"in(A2) && in(B2) && in(C2)"];

    SHPStateChartGuard *guardD2ToD1 = [SHPStateChartGuard guardWithCondition:^BOOL {
        return SA.activeState != SA2 || SB.activeState != SB2 || SC.activeState != SC2;
    } guardDescription:@"!in(A2) || !in(B2) || !in(C2)"];

    [SD1 transitionTo:SD2 guard:guardD1ToD2];
    [SD2 transitionTo:SD1 guard:guardD2ToD1];
}

@end
