// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.


#import "Scenario9StateChart.h"
#import "Submachine1StateChart.h"

BASE_STATE(Scenario9StateChart)

STATE(Scenario9StateChart, State1)STATE_END
AND_STATE(Scenario9StateChart, Submachines)STATE_END

STATE(Scenario9StateChart, SubmachineState1)STATE_END
STATE(Scenario9StateChart, SubmachineState2)STATE_END
STATE(Scenario9StateChart, SubmachineState3)STATE_END

STATE(Scenario9StateChart, StateRegion1)STATE_END
STATE(Scenario9StateChart, StateRegion2)STATE_END
STATE(Scenario9StateChart, StateRegion3)STATE_END


@implementation Scenario9StateChart

- (id)init {
    if (!(self = [super init])) {return nil;}

    _eventInPort = [self inputPortWithProtocol:@protocol(Scenario9StateChartEventProtocol) name:@"Event In"];

    return self;
}

- (void)configureWithConfiguration:(id <SHPStateChartConfigurationProtocol>)configuration {
    State1 *s1 = [State1 new];
    SubmachineState1 *submachineState1 = [SubmachineState1 new];
    SubmachineState2 *submachineState2 = [SubmachineState2 new];
    SubmachineState3 *submachineState3 = [SubmachineState3 new];

    StateRegion1 *stateRegion1 = [StateRegion1 new];
    StateRegion2 *stateRegion2 = [StateRegion2 new];
    StateRegion3 *stateRegion3 = [StateRegion3 new];


    Submachine1StateChart *sm1 = [[Submachine1StateChart alloc] initWithMagicConstant:0 event3ExitState:submachineState1];
    Submachine1StateChart *sm2 = [[Submachine1StateChart alloc] initWithMagicConstant:1 event3ExitState:submachineState2];
    Submachine1StateChart *sm3 = [[Submachine1StateChart alloc] initWithMagicConstant:2 event3ExitState:submachineState3];

    Submachines *submachines = [Submachines new];
    [submachines setRegions:@[stateRegion1, stateRegion2, stateRegion3]];
    [stateRegion1 setSubStates:@[sm1.sm1, submachineState1] initialState:(id) sm1.sm1];
    [stateRegion2 setSubStates:@[sm2.sm1, submachineState2] initialState:(id) sm2.sm1];
    [stateRegion3 setSubStates:@[sm3.sm1, submachineState3] initialState:(id) sm3.sm1];
    [configuration.rootState setSubStates:@[s1, submachines] initialState:s1];

    [s1 onEvent:@selector(event2) transitionTo:submachines];
    [submachines onEvent:@selector(event2) transitionTo:s1];
    [s1 onEvent:@selector(event3) transitionToStates:@[sm1.sm1b, sm2.sm1b, sm3.sm1b] guard:nil];
    [submachineState1 onEvent:@selector(event3) transitionTo:(id) sm1.sm1];
    [submachineState2 onEvent:@selector(event3) transitionTo:(id) sm2.sm1];
    [submachineState3 onEvent:@selector(event3) transitionTo:(id) sm3.sm1];
}

@end
