// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <Foundation/Foundation.h>

@interface NSArray (GroupingAdditions)

/// ---------------------------------------------------------------------
/// @name Getting objects in the array.
/// ---------------------------------------------------------------------

/// Returns the first object in the array or nil if empty.
-(id)firstObject;

/// ---------------------------------------------------------------------
/// @name Grouping objects in the array
/// ---------------------------------------------------------------------

/// Groups the array by some property on the object in the array
/// The block should return the property that the objects should be grouped by.
-(NSArray *)groupByPropertyFromBlock:(id (^)(id))propertyBlock;

/// Groups the array by some property on the object in the array
/// The block should return the property that the objects should be grouped by.
/// This is a variant of groupByPropertyFromBlock that returns a dict instead of an array.
-(NSDictionary *)dictionaryGroupedByPropertyFromBlock:(id (^)(id))propertyBlock;

/// ---------------------------------------------------------------------
/// @name Flattening the array hierarchy
/// ---------------------------------------------------------------------

// Flattens the array hierarchy so that any nested arrays will have their
// objects included in the outer array
-(NSArray *)arrayByFlattening;

@end