// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.


#import "TikTokStateChart.h"
#import "SHPStateChartGuard.h"
#import "SHPStateChart.h"
#import "SHPEventMessage.h"

BASE_STATE(TikTokStateChart)

#pragma mark - StateStart
STATE(TikTokStateChart, StateStart)STATE_END

#pragma mark - StateTikTok
STATE(TikTokStateChart, StateTikTok)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    NSLog(@"Starting count");
    self.stateChart.startTime = CACurrentMediaTime();
}

- (void)completedToks {
    self.stateChart.timeInterval = CACurrentMediaTime() - self.stateChart.startTime;
    CFTimeInterval ti = self.stateChart.timeInterval;
    NSLog(@"%f sec / %i msg", ti, self.stateChart.goal);
    NSLog(@"%.1f msg / sec", self.stateChart.goal / ti);
}

STATE_END

#pragma mark - StateTik
STATE(TikTokStateChart, StateTik)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.eventPort performTik];
}


STATE_END

#pragma mark - StateTok
STATE(TikTokStateChart, StateTok)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.eventPort performTok];
}

- (void)performTok {
    self.stateChart.numberOfToks++;
}

STATE_END

#pragma mark - StateDone
STATE(TikTokStateChart, StateDone)STATE_END

@implementation TikTokStateChart

- (id)initWithGoal:(NSUInteger)goal {
    _goal = goal;
    if (!(self = [super init])) {return nil;}

    return self;
}

- (void)configureWithConfiguration:(SHPStateChartConfiguration *)configuration {
    configuration.eventHandlerProtocol = @protocol(TikTokStateChartEventProtocol);

    StateStart *start = [StateStart new];
    StateTikTok *tikTok = [StateTikTok new];
    StateTik *tik = [StateTik new];
    StateTok *tok = [StateTok new];
    [tikTok setSubStates:@[tik, tok] initialState:tik];

    StateDone *done = [StateDone new];
    [configuration.rootState setSubStates:@[start, tikTok, done] initialState:start];

    [start transitionTo:tikTok];
    [tik onEvent:@selector(performTik) transitionTo:tok];
    [tok onEvent:@selector(performTok) transitionTo:tik];
    [tikTok onEvent:@selector(completedToks) transitionTo:done];
    SHPStateChartGuard *tokGuard = [SHPStateChartGuard guardWithCondition:^BOOL { return self.numberOfToks == _goal; } guardDescription:[NSString stringWithFormat:@"numberOfToks == %i", _goal]];
    [tikTok transitionTo:done action:@selector(completedToks) guard:tokGuard];
}

@end
