// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import "SHPMessageClientBridgeComponent.h"
#import "SHPProxyInputPort.h"
#import "SHPProxyOutputPort.h"
#import "SHPMessageRouter.h"


@implementation SHPMessageClientBridgeComponent {

}

- (id)init {
    self = [super init];
    if (self) {
        _commandInputPort = [self proxyInputPortWithProtocol:@protocol(SHPMessageClientBridgeCommandProtocol) name:@"Command"];
        _notificationOutputPort = [self proxyOutputPortWithProtocol:@protocol(SHPMessageClientBridgeNotificationProtocol) name:@"Notification"];

        _messagesInputPort = [self proxyInputPortWithProtocol:@protocol(SHPMessageClientBridgeMessageInputProtocol) name:@"Message in"];
        _messagesOutputPort = [self proxyOutputPortWithProtocol:@protocol(SHPMessageClientBridgeMessageOutputProtocol) name:@"Message out"];
    }

    return self;
}

- (void)configure {
    SHPMessageClientBridgeStateChart *messageClientBridgeController = [[SHPMessageClientBridgeStateChart alloc] init];
    SHPBonjourBrowserStateChart *bonjourBrowserStateChart = [[SHPBonjourBrowserStateChart alloc] initWithDomain:@"local" type:@"_shapeapp._tcp"];
    SHPSocketClientStateChart *socketClientStateChart = [[SHPSocketClientStateChart alloc] init];

    [self addSubComponent:messageClientBridgeController];
    [self addSubComponent:bonjourBrowserStateChart];
    [self addSubComponent:socketClientStateChart];

    messageClientBridgeController.commandOutputPort.channel = bonjourBrowserStateChart.commandInputPort.channel = socketClientStateChart.commandInputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];
    messageClientBridgeController.notificationInputPort.channel = bonjourBrowserStateChart.dataOutputPort.channel = socketClientStateChart.notificationOutputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];

    self.commandInputPort.proxyChannel = messageClientBridgeController.commandInputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];
    self.notificationOutputPort.proxyChannel = messageClientBridgeController.notificationOutputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];

    self.messagesInputPort.proxyChannel = messageClientBridgeController.messageInputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];
    self.messagesOutputPort.proxyChannel = messageClientBridgeController.messageOutputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];

    [messageClientBridgeController initialTransition];
    [bonjourBrowserStateChart initialTransition];
    [socketClientStateChart initialTransition];
}

@end