// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import "SHPMessageClientBridgeStateChart.h"
#import "SHPEventMessage.h"
#import "SHPSocketConnectionInfo.h"
#import "SHPWrappedMessage.h"


@interface SHPMessageClientBridgeStateChart ()
@property(nonatomic, strong) SHPSocketConnectionInfo *connectionInfo;
@end

@protocol SHPMessageClientBridgeStateChartEventProtocol <SHPMessageClientBridgeInputCommandProtocol,SHPMessageClientBridgeInputNotificationProtocol, SHPMessageClientBridgeMessageInputProtocol>
@optional
@end

BASE_STATE(SHPMessageClientBridgeStateChart)

#pragma mark - StateStopped
STATE(SHPMessageClientBridgeStateChart, StateStopped)STATE_END

#pragma mark - StateRunning
STATE(SHPMessageClientBridgeStateChart, StateRunning)STATE_END

#pragma mark - StateBrowsing
STATE(SHPMessageClientBridgeStateChart, StateBrowsing)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.commandOutputPort eventBonjourBrowserSearch];
}

- (void)eventBonjourBrowserFoundServiceWithConnectionInfo:(SHPSocketConnectionInfo *)connectionInfo {
    self.stateChart.connectionInfo = connectionInfo;
}

STATE_END

#pragma mark - StateFoundService
STATE(SHPMessageClientBridgeStateChart, StateFoundService)STATE_END

#pragma mark - StateStoppingBrowser
STATE(SHPMessageClientBridgeStateChart, StateStoppingBrowser)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.commandOutputPort eventBonjourBrowserStop];
}

STATE_END

#pragma mark - StateDisconnecting
STATE(SHPMessageClientBridgeStateChart, StateDisconnecting)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.commandOutputPort eventSocketClientDisconnect];
}

STATE_END

#pragma mark - StateConnecting
STATE(SHPMessageClientBridgeStateChart, StateConnecting)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.commandOutputPort eventSocketClientConnectWithConnectionInfo:self.stateChart.connectionInfo];
}

STATE_END

#pragma mark - StateConnected
STATE(SHPMessageClientBridgeStateChart, StateConnected)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
}


- (void)eventMessageClientBridgeSendMessage:(SHPWrappedMessage *)message {
    [self.stateChart.commandOutputPort eventSocketClientSendObject:(id <NSCoding>) message];
}

- (void)eventSocketClientReceivedObject:(id <NSCoding>)object {
    [self.stateChart.messageOutputPort eventMessageClientBridgeReceivedMessage:(id) object];
}

STATE_END

#pragma mark - StateStopping
STATE(SHPMessageClientBridgeStateChart, StateStopping)STATE_END


@implementation SHPMessageClientBridgeStateChart {

}

- (id)init {
    self = [super init];
    if (self) {
        _commandInputPort = [self inputPortWithProtocol:@protocol(SHPMessageClientBridgeInputCommandProtocol) name:@"Command"];
        _commandOutputPort = [self outputPortWithProtocol:@protocol(SHPMessageClientBridgeOutputCommandProtocol) name:@"Internal command"];
        _notificationInputPort = [self inputPortWithProtocol:@protocol(SHPMessageClientBridgeInputNotificationProtocol) name:@"Notification"];
        _notificationOutputPort = [self outputPortWithProtocol:@protocol(SHPMessageClientBridgeOutputNotificationProtocol) name:@"Internal notification"];
        _messageInputPort = [self inputPortWithProtocol:@protocol(SHPMessageClientBridgeMessageInputProtocol) name:@"Message in"];
        _messageOutputPort = [self outputPortWithProtocol:@protocol(SHPMessageClientBridgeMessageOutputProtocol) name:@"Message out"];
    }

    return self;
}

- (void)configureWithConfiguration:(id <SHPStateChartConfigurationProtocol>)configuration {
	StateStopped *stateStopped = [StateStopped new];
	StateRunning *stateRunning = [StateRunning new];
	StateBrowsing *stateBrowsing = [StateBrowsing new];
	StateFoundService *stateFoundService = [StateFoundService new];
	StateStoppingBrowser *stateStoppingBrowser = [StateStoppingBrowser new];
    StateDisconnecting *stateDisconnecting = [StateDisconnecting new];
	StateConnecting *stateConnecting = [StateConnecting new];
	StateConnected *stateConnected = [StateConnected new];
    StateStopping *stateStopping = [StateStopping new];

    [stateRunning setSubStates:@[stateBrowsing,stateFoundService] initialState:stateBrowsing];
    [stateFoundService setSubStates:@[stateConnecting, stateConnected] initialState:stateConnecting];
    [stateStopping setSubStates:@[stateStoppingBrowser, stateDisconnecting] initialState:stateDisconnecting];
    [configuration.rootState setSubStates:@[stateStopped, stateRunning, stateStopping] initialState:stateStopped];

	[stateStopped onEvent:@selector(eventMessageClientBridgeStart) transitionTo:stateRunning];
	[stateBrowsing onEvent:@selector(eventBonjourBrowserFoundServiceWithConnectionInfo:) transitionTo:stateFoundService];
    [stateBrowsing onEvent:@selector(eventMessageClientBridgeStop) transitionTo:stateStoppingBrowser];
    [stateFoundService onEvent:@selector(eventMessageClientBridgeStop) transitionTo:stateStopping];
	[stateConnecting onEvent:@selector(eventSocketClientConnected) transitionTo:stateConnected];
	[stateRunning onEvent:@selector(eventSocketClientDisconnected) transitionTo:stateBrowsing];
    [stateConnected internalTransitionOnEvent:@selector(eventSocketClientReceivedObject:)];
    [stateConnected internalTransitionOnEvent:@selector(eventMessageClientBridgeSendMessage:)];
    [stateDisconnecting onEvent:@selector(eventSocketClientDisconnected) transitionTo:stateStoppingBrowser];
    [stateStoppingBrowser onEvent:@selector(eventBonjourBrowserStopped) transitionTo:stateStopped];

    [configuration setEventHandlerProtocol:@protocol(SHPMessageClientBridgeStateChartEventProtocol)];
}

@end
