// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import "SHPSocketServerStateChart.h"
#import "SHPBonjourServerStateChart.h"
#import "SHPBonjourBrowserStateChart.h"
#import "SHPMessageServerBridgeComponent.h"
#import "SHPMessageRouter.h"
#import "SHPProxyOutputPort.h"
#import "SHPProxyInputPort.h"


@implementation SHPMessageServerBridgeComponent {

}

- (id)init {
    self = [super init];
    if (self) {
        _commandInputPort = [self proxyInputPortWithProtocol:@protocol(SHPMessageServerBridgeComponentCommandProtocol) name:@"Command"];
        _notificationOutputPort = [self proxyOutputPortWithProtocol:@protocol(SHPMessageServerBridgeComponentNotificationProtocol) name:@"Notification"];

        _messagesInputPort = [self proxyInputPortWithProtocol:@protocol(SHPMessageServerBridgeMessageInputProtocol) name:@"Message in"];
        _messagesOutputPort = [self proxyOutputPortWithProtocol:@protocol(SHPMessageServerBridgeMessageOutputProtocol) name:@"Message out"];
    }

    return self;
}

- (void)configure {
    uint16_t port = 10000;
    SHPMessageServerBridgeStateChart *messageServerBridgeController = [[SHPMessageServerBridgeStateChart alloc] init];
    SHPBonjourServerStateChart *bonjourServerStateChart = [[SHPBonjourServerStateChart alloc] initWithDomain:@"" type:@"_shapeapp._tcp" name:@"" port:port];
    SHPSocketServerStateChart *socketServerStateChart = [[SHPSocketServerStateChart alloc] initWithPort:port interface:@""];

    messageServerBridgeController.commandOutputPort.channel = bonjourServerStateChart.commandInputPort.channel = socketServerStateChart.commandInputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];
    messageServerBridgeController.notificationInputPort.channel = bonjourServerStateChart.notificationOutputPort.channel = socketServerStateChart.notificationOutputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];

    self.commandInputPort.proxyChannel = messageServerBridgeController.commandInputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];
    self.notificationOutputPort.proxyChannel = messageServerBridgeController.notificationOutputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];

    self.messagesInputPort.proxyChannel = messageServerBridgeController.messageInputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];
    self.messagesOutputPort.proxyChannel = messageServerBridgeController.messageOutputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];

    [messageServerBridgeController initialTransition];
    [bonjourServerStateChart initialTransition];
    [socketServerStateChart initialTransition];

    [self addSubComponent:messageServerBridgeController];
    [self addSubComponent:bonjourServerStateChart];
    [self addSubComponent:socketServerStateChart];

}

@end