// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.


#import <SHPStateChart/SHPMessageRouter.h>
#import "SHPBonjourBrowserStateChart.h"
#import "SHPSocketServerStateChart.h"
#import "SHPBonjourServerStateChart.h"
#import "SHPMessageServerBridgeStateChart.h"
#import "SHPEventMessage.h"
#import "SHPWrappedMessage.h"


@protocol SHPMessageServerBridgeStateChartEventProtocol <SHPMessageServerBridgeCommandInputProtocol, SHPMessageServerBridgeNotificationInputProtocol, SHPMessageServerBridgeMessageInputProtocol>
@optional
- (void)eventMessageServerBridgeStop;
@end

BASE_STATE(SHPMessageServerBridgeStateChart)

#pragma mark - StateStopped
STATE(SHPMessageServerBridgeStateChart, StateStopped)STATE_END

#pragma mark - StateWorking
STATE(SHPMessageServerBridgeStateChart, StateWorking)STATE_END

#pragma mark - StateStartingSocketServer
STATE(SHPMessageServerBridgeStateChart, StateStartingSocketServer)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.commandOutputPort eventSocketServerStart];
}

STATE_END

#pragma mark - StateStopping
STATE(SHPMessageServerBridgeStateChart, StateStopping)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.commandOutputPort eventSocketServerStop];
}

STATE_END

#pragma mark - StateStoppingSocketServer
STATE(SHPMessageServerBridgeStateChart, StateStoppingSocketServer)STATE_END

#pragma mark - StateStoppingBroadcastingService
STATE(SHPMessageServerBridgeStateChart, StateStoppingBroadcastingService)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.commandOutputPort eventBonjourServerStop];
}

STATE_END

#pragma mark - StateStartingBroadcastingService
STATE(SHPMessageServerBridgeStateChart, StateStartingBroadcastingService)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [self.stateChart.commandOutputPort eventBonjourServerPublish];
}

STATE_END

#pragma mark - StateBroadcastingService
STATE(SHPMessageServerBridgeStateChart, StateBroadcastingService)

- (void)eventEntry:(SHPEventMessage *)eventMessage {
    [super eventEntry:eventMessage];
}


- (void)eventMessageServerBridgeSendMessage:(SHPWrappedMessage *)message {
    [self.stateChart.commandOutputPort eventSocketServerSendObject:(id <NSCoding>) message];
}

- (void)eventSocketServerReceivedObject:(id)object {
    [self.stateChart.messageOutputPort eventMessageServerBridgeReceivedMessage:object];
}

STATE_END


@implementation SHPMessageServerBridgeStateChart {

}

- (id)init {
    self = [super init];
    if (self) {
        _commandInputPort = [self inputPortWithProtocol:@protocol(SHPMessageServerBridgeCommandInputProtocol) name:@"Command"];
        _commandOutputPort = [self outputPortWithProtocol:@protocol(SHPMessageServerBridgeCommandOutputProtocol) name:@"Internal command"];
        _notificationInputPort = [self inputPortWithProtocol:@protocol(SHPMessageServerBridgeNotificationInputProtocol) name:@"Notification"];
        _notificationOutputPort = [self outputPortWithProtocol:@protocol(SHPMessageServerBridgeNotificationOutputProtocol) name:@"Internal notification"];

        _messageInputPort = [self inputPortWithProtocol:@protocol(SHPMessageServerBridgeMessageInputProtocol) name:@"Message in"];
        _messageOutputPort = [self outputPortWithProtocol:@protocol(SHPMessageServerBridgeMessageOutputProtocol) name:@"Message out"];
    }

    return self;
}

- (void)configureWithConfiguration:(id <SHPStateChartConfigurationProtocol>)configuration {
    //init states
	StateStopped *stateStopped = [StateStopped new];
	StateWorking *stateWorking = [StateWorking new];
    StateStartingSocketServer *stateStartingSocketServer = [StateStartingSocketServer new];
	StateStopping *stateStopping = [StateStopping new];
    StateStoppingSocketServer *stateStoppingSocketServer = [StateStoppingSocketServer new];
    StateStoppingBroadcastingService *stateStoppingBroadcastingService = [StateStoppingBroadcastingService new];
    StateStartingBroadcastingService *stateStartingBroadcastingService = [StateStartingBroadcastingService new];
    StateBroadcastingService *stateBroadcastingService = [StateBroadcastingService new];

    //Setup sub states
    [stateWorking setSubStates:@[stateStartingSocketServer,stateStopping, stateStartingBroadcastingService,stateBroadcastingService] initialState:stateStartingSocketServer];
    [stateStopping setSubStates:@[stateStoppingSocketServer,stateStoppingBroadcastingService] initialState:stateStoppingSocketServer];
    [configuration.rootState setSubStates:@[stateStopped, stateWorking] initialState:stateStopped];

    //Setup transitions
	[stateStopped onEvent:@selector(eventMessageServerBridgeStart) transitionTo:stateWorking];
    [stateWorking onEvent:@selector(eventMessageServerBridgeStop) transitionTo:stateStopping];
    [stateStoppingSocketServer onEvent:@selector(eventSocketServerStopped) transitionTo:stateStoppingBroadcastingService];
    [stateStoppingBroadcastingService onEvent:@selector(eventBonjourServerStopped) transitionTo:stateStopped];
    [stateStartingSocketServer onEvent:@selector(eventSocketServerStarted) transitionTo:stateStartingBroadcastingService];
    [stateStartingBroadcastingService onEvent:@selector(eventBonjourServerPublished) transitionTo:stateBroadcastingService];

    [stateBroadcastingService internalTransitionOnEvent:@selector(eventMessageServerBridgeSendMessage:)];

    [stateBroadcastingService internalTransitionOnEvent:@selector(eventMessageServerBridgeReceivedMessage:)];

    //Configure
    [configuration setEventHandlerProtocol:@protocol(SHPMessageServerBridgeStateChartEventProtocol)];

}

@end
