// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <SHPStateChart/SHPMessageClientBridgeStateChart.h>
#import <SHPStateChart/SHPEventMessage.h>
#import <objc/runtime.h>
#import "SHPMessageUnwrapperComponent.h"
#import "SHPWrappedMessage.h"


@implementation SHPMessageUnwrapperComponent {

}

- (id)initWithProtocol:(Protocol *)protocol channelName:(NSString *)channelName {
    self = [super init];
    if (self) {
        _channelName = channelName;
        _protocol = protocol;
        _inputPort = [self inputPortWithProtocol:@protocol(SHPMessageUnwrapperInputProtocol) name:@"Input"];
        _outputPort = [self outputPortWithProtocol:protocol name:@"Output"];
    }

    return self;
}

- (void)messageUnwrapperUnwrapMessage:(SHPWrappedMessage *)wrappedMessage {
    if([wrappedMessage.channelName isEqualToString:_channelName]) {
        SEL selector = NSSelectorFromString(wrappedMessage.selectorName);
        NSMethodSignature *signature =  [self methodSignatureForSelector:selector];

        if(signature) {
            NSInvocation *invocation = [NSInvocation invocationWithMethodSignature:signature];
            NSUInteger reservedArguments = 2;

            for (NSUInteger i = 0; i < wrappedMessage.arguments.count; i++) {
                id arg = [wrappedMessage.arguments objectAtIndex:i];
                [invocation setArgument:&arg atIndex:i+reservedArguments];

            }

            [invocation setSelector:selector];
            [invocation setTarget:_outputPort];
            [invocation invoke];
        }
    }
}

- (NSMethodSignature *)methodSignatureForSelector:(SEL)inSelector {
    NSAssert(self.protocol!= nil, @"Protocol must be specified");
    struct objc_method_description methodDescription = protocol_getMethodDescription(self.protocol,inSelector, YES, YES);
    if (!methodDescription.name) {
        methodDescription = protocol_getMethodDescription(self.protocol,inSelector, NO, YES);
    }

    if (!methodDescription.name) {
        return nil;
    }

    NSMethodSignature *methodSignature = [NSMethodSignature signatureWithObjCTypes:methodDescription.types];

    return methodSignature;
}

@end