// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <SHPStateChart/SHPEventMessage.h>
#import <SHPStateChart/SHPInputPort.h>
#import "SHPMessageWrapperComponent.h"
#import "SHPWrappedMessage.h"


@implementation SHPMessageWrapperComponent {

}

- (id)initWithChannelName:(NSString *)channelName {
    self = [super init];
    if (self) {
        _channelName = channelName;
        _inputPort = [self inputPortWithProtocol:@protocol(SHPMessageWrapperInputProtocol) name:@"Input"];
        _outputPort = [self outputPortWithProtocol:@protocol(SHPMessageWrapperOutputProtocol) name:@"Output"];
    }

    return self;
}

- (void)inputPort:(SHPInputPort *)inputPort didReceiveMessage:(SHPEventMessage *)message {
    if([message isKindOfClass:[SHPEventMessage class]]) {
        NSInvocation *invocation = message.invocation;
        NSMutableArray *arguments = [NSMutableArray new];
        NSUInteger numberOfArguments = invocation.methodSignature.numberOfArguments;
        for (NSUInteger index = 2; index < numberOfArguments; index++) {
            __unsafe_unretained id arg = nil;
            [invocation getArgument:&arg atIndex:index];
            [arguments addObject:arg];
        }

        SHPWrappedMessage *wrappedMessage = [[SHPWrappedMessage alloc] initWithArguments:arguments selector:invocation.selector channelName:_channelName];
        [self.outputPort channelWrapperWrappedMessage:wrappedMessage];
    }
}

@end