// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import "NSArray+GroupingAdditions.h"


@implementation NSArray (GroupingAdditions)

-(id)firstObject {
	if ([self count] == 0)
		return nil;
	else
		return [self objectAtIndex:0];
}

- (NSArray *)groupByPropertyFromBlock:(id(^)(id obj))propertyBlock {
	// a dict is used to group the array contents in containers based on the property passed to this method
	NSMutableDictionary *groupedDict = [[NSMutableDictionary alloc] initWithCapacity:[self count]];

	// loop through each object in the array and add to the dict
	// the property is used as the key in the dict
	[self enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
		// the key is the value returned by the property
		id key = propertyBlock(obj);
		NSAssert(key, @"propertyBlock must supply a key");
		// if no entry exists in the dict for that key then set up a mutable array for the key
		if (![groupedDict objectForKey:key]) {
			[groupedDict setObject:[NSMutableArray array] forKey:key];
		}

		// add the object to the dict
		[[groupedDict objectForKey:key] addObject:obj];
	}];

	// create immutable results array
	NSArray *groupedResultArray = [groupedDict allValues];

	// release stuff

	return groupedResultArray;
}

- (NSDictionary *)dictionaryGroupedByPropertyFromBlock:(id(^)(id obj))propertyBlock {
	// a dict is used to group the array contents in containers based on the property passed to this method
	NSMutableDictionary *groupedDict = [[NSMutableDictionary alloc] initWithCapacity:[self count]];

	// loop through each object in the array and add to the dict
	// the property is used as the key in the dict
	[self enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
		// the key is the value returned by the property
		id key = propertyBlock(obj);
		NSAssert(key, @"propertyBlock must supply a key");
		// if no entry exists in the dict for that key then set up a mutable array for the key
		if (![groupedDict objectForKey:key]) {
			[groupedDict setObject:[NSMutableArray array] forKey:key];
		}

		// add the object to the dict
		[[groupedDict objectForKey:key] addObject:obj];
	}];
	return [groupedDict copy];
}

- (NSArray *)arrayByFlattening {
	NSMutableArray *flattenedArray = [NSMutableArray arrayWithCapacity:[self count]];

	for (id obj in self) {
		if ([obj isKindOfClass:[NSArray class]]) {
			[flattenedArray addObjectsFromArray:[(NSArray *)obj arrayByFlattening]];
		} else {
			[flattenedArray addObject:obj];
		}
	}

	return [flattenedArray copy];
}

@end