// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import "MessageTimer.h"
#import "SHPUtilities.h"

@implementation MessageTimer {
    NSMutableDictionary *_timers;
    dispatch_queue_t _queue;
}

#pragma mark - Init / dealloc

- (id)init {
    if (!(self = [super init])) {
        return nil;
    }

    _timers = [NSMutableDictionary dictionary];
    _queue = SHPCreateSerialQueueWithRandomName();

    return self;
}



#pragma mark - Timers

- (void)addTimerWithMessage:(id)msg component:(id <SHPChannelDelegate>)component interval:(NSTimeInterval)interval tag:(NSString *)tag repeats:(BOOL)repeats {
    __block dispatch_source_t timer = SHPCreateDispatchTimer(interval, 0.0f, _queue, ^{
        [component channel:nil didDeliverMessage:msg];

        if (!repeats) {
            dispatch_source_cancel(timer);
        }
    });

    dispatch_async(_queue, ^{

        NSValue *timerWrapper = [NSValue valueWithPointer:(__bridge const void *)(timer)];
        [_timers setObject:timerWrapper forKey:tag];
    });
}

- (void)removeTimerWithTag:(NSString *)tag {
    dispatch_async(_queue, ^{
        NSValue *timerWrapper = [_timers objectForKey:tag];
        if (timerWrapper) {
            dispatch_source_t timer = [timerWrapper pointerValue];
            dispatch_source_cancel(timer);
            [_timers removeObjectForKey:tag];
        }
    });
}

#pragma mark - Singleton

+ (MessageTimer *)sharedInstance {
    static MessageTimer *sharedInstance = nil;
    static dispatch_once_t pred;

    if (sharedInstance) return sharedInstance;

    dispatch_once(&pred, ^{
        sharedInstance = [MessageTimer alloc];
        sharedInstance = [sharedInstance init];
    });

    return sharedInstance;
}

@end
