// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <SHPStateChart/SHPComponentRegister.h>
#import <SHPStateChart/SHPProxyOutputPort.h>
#import "SHPComponent.h"
#import "SHPUtilities.h"
#import "SHPProxyInputPort.h"

@implementation SHPComponent {
    NSString *_identifier;
    NSMutableArray *_ports;
    NSMutableArray *_subComponents;
}

#pragma mark - Lifecycle

- (id)init {
    return [self initWithComponentRegisterChannel:[[SHPComponentRegister sharedInstance] commandInputPort].channel];
}

- (id)initWithComponentRegisterChannel:(SHPChannel *)componentRegisterChannel {
    return [self initWithComponentRegisterChannel:componentRegisterChannel name:NSStringFromClass([self class])];
}

- (id)initWithComponentRegisterChannel:(SHPChannel *)componentRegisterChannel name:(NSString *)name {
    if (!(self = [super init])) { return nil; }

    _componentRegisterPort = [self outputPortWithProtocol:@protocol(SHPComponentRegisterCommandProtocol) name:@"Register"];
    _componentRegisterPort.channel= componentRegisterChannel;
    _ports = [NSMutableArray new];
    _subComponents = [NSMutableArray new];
    _messageQueue = self.defaultMessageQueue;
    _identifier = [self componentIdentifier];
    _name = name;
    _portTraceDictionary = [NSMutableDictionary new];

    [self.componentRegisterPort componentRegisterAddComponent:self];

    return self;
}

#pragma mark - Private getters

- (NSString *)componentIdentifier {
    return SHPCreateUDID();;
}

#pragma mark - Public getters

- (dispatch_queue_t)defaultMessageQueue {
    NSString *queueName = [self messageQueueName];
    dispatch_queue_t messageQueue = dispatch_queue_create([queueName cStringUsingEncoding:NSASCIIStringEncoding], DISPATCH_QUEUE_SERIAL);

    return messageQueue;
}

- (NSString *)messageQueueName {
    NSString *uuidString = SHPCreateUDID();
    return [NSString stringWithFormat:@"%@-%@", NSStringFromClass(self.class),uuidString];
}

#pragma mark - Ports

- (id)inputPortWithProtocol:(Protocol *)protocol name:(NSString *)name {
    SHPInputPort *port = [[SHPInputPort alloc] initWithOwner:self protocol:protocol name:name];
    [_ports addObject:port];
    return port;
}

- (id)outputPortWithProtocol:(Protocol *)protocol name:(NSString *)name {
    SHPOutputPort *port = [[SHPOutputPort alloc] initWithOwner:self protocol:protocol name:name];
    [_ports addObject:port];
    return port;
}

- (id)proxyInputPortWithProtocol:(Protocol *)protocol name:(NSString *)name {
    SHPProxyInputPort *port = [[SHPProxyInputPort alloc] initWithOwner:self protocol:protocol name:name];
    [_ports addObject:port];
    return port;
}

- (id)proxyOutputPortWithProtocol:(Protocol *)protocol name:(NSString *)name {
    SHPProxyOutputPort *port = [[SHPProxyOutputPort alloc] initWithOwner:self protocol:protocol name:name];
    [_ports addObject:port];
    return port;
}

#pragma mark - Sub component

- (void)addSubComponent:(SHPComponent *)subComponent {
    [_subComponents addObject:subComponent];
    subComponent.superComponent = self;
}

#pragma mark - Port delegate

- (void)inputPort:(SHPInputPort *)inputPort didReceiveMessage:(SHPEventMessage *)message {
    NSInvocation *invocation = message.invocation;
    if([self respondsToSelector:invocation.selector]) {
        [invocation setTarget:self];
        [invocation invoke];
    }
}


@end
