// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <SHPStateChart/SHPComponentRegister.h>
#import <SHPStateChart/SHPInputPort.h>
#import <SHPStateChart/SHPOutputPort.h>
#import "SHPMessageRouter.h"

@interface SHPComponentRegister ()

@property(strong) NSMutableDictionary *components;

@end

@implementation SHPComponentRegister

- (id)init {
    if (!(self = [super initWithComponentRegisterChannel:nil])) { return nil; }

    self.components = [NSMutableDictionary new];
    [_components setObject:self forKey:self.identifier];
    _commandInputPort = [self inputPortWithProtocol:@protocol(SHPComponentRegisterCommandProtocol) name:@"Command"];
    _commandInputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];
    _notificationOutputPort = [self outputPortWithProtocol:@protocol(SHPComponentRegisterNotificationProtocol) name:@"Notification"];
    _notificationOutputPort.channel = [[SHPMessageRouter sharedInstance] publishSubscribeChannel];

    return self;
}

+ (SHPComponentRegister *)sharedInstance {
    static dispatch_once_t once;
    static SHPComponentRegister *sharedFoo;
    dispatch_once(&once, ^ { sharedFoo = [[self alloc] init]; });
    return sharedFoo;
}

- (NSString *)messageQueueName {
    return @"ComponentRegister";
}

- (void)componentRegisterAddComponent:(SHPComponent *)component {
    [_components setObject:component forKey:component.identifier];
    [self componentRegisterGetComponents];
}

- (void)componentRegisterRemoveComponent:(SHPComponent *)component {
    [_components removeObjectForKey:component.identifier];
    [self componentRegisterGetComponents];
}

- (void)componentRegisterGetComponents {
    [self.notificationOutputPort componentRegisterComponents:_components.allValues];
}

@end