// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <SHPStateChart/SHPState.h>
#import <SHPStateChart/SHPEventRepresentation.h>
#import "SHPEventRepresentation.h"
#import "SHPStateRepresentation.h"


@implementation SHPEventRepresentation {

}

- (instancetype)initWithEventType:(SHPEventRepresentationEventType)eventType transitionAction:(NSString *)transitionAction stateRepresentation:(SHPStateRepresentation *)stateRepresentation {
    self = [super init];
    if (self) {
        _eventType = eventType;
        _eventTransitionAction = transitionAction;
        _stateRepresentation = stateRepresentation;
    }

    return self;
}

- (NSString *)description {
    NSMutableString *description = [NSMutableString stringWithFormat:@"<%@: ", NSStringFromClass([self class])];
    [description appendFormat:@"eventType=%@", [self stringFromEventType:_eventType]];
    if (_eventTransitionAction) { [description appendFormat:@", eventTransitionAction=%@", self.eventTransitionAction]; }
    [description appendFormat:@", state=%@", self.stateRepresentation];
    [description appendString:@">"];
    return description;
}

- (NSString *)eventTypeString {
    return [self stringFromEventType:_eventType];
}

- (NSObject *)stringFromEventType:(SHPEventRepresentationEventType)type {
    switch (type) {
        case SHPEventRepresentationEventTypeInitial:
            return @"Initial";
        case SHPEventRepresentationEventTypeExit:
            return @"Exit";
        case SHPEventRepresentationEventTypeEntry:
            return @"Entry";
        case SHPEventRepresentationEventTypeAction:
            return @"Action";
        default:
            return @"<UNKNOWN EVENT TYPE>";
    }
}

@end
