// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import <Foundation/Foundation.h>

@class SHPStateChart;
@class SHPEventMessage;
@class SHPStateChartTransition;
@class SHPStateChartGuard;
@class SHPEventHandlingResult;
@class SHPStateChartGuardTransition;
@class SHPOrStateRepresentation;
@class SHPStateRepresentation;
@class SHPTransitionRepresentation;

typedef enum {
    HistoryModeNone,
    HistoryModeShallow,
    HistoryModeDeep,
} HistoryMode;

@interface SHPState : NSObject {
@protected NSMutableDictionary *_events;
@protected SHPState *_historyState;
@protected SHPStateChart *_stateChart;
}

@property (nonatomic, readonly) SHPStateChart *stateChart;
@property (nonatomic) SHPState *superstate;
@property (nonatomic, readonly) NSMutableDictionary *events;

- (NSString *)name;

// Statechart
- (void)setStateChartUnlessSetRecursively:(SHPStateChart *)statechart;

// State
- (SHPState *)findFirstStateOfClassRecursively:(Class)stateClass;

// Events
- (void)eventEntry:(SHPEventMessage *)eventMessage;
- (void)eventExit:(SHPEventMessage *)eventMessage;
- (void)eventInitial:(SHPEventMessage *)eventMessage;

// Transition
- (NSOrderedSet *)transitionsRespondingToEventMessage:(SHPEventMessage *)eventMessage currentPerformingTransition:(SHPStateChartTransition *)currentPerformingTransition;
- (BOOL)isOrContainsTargetStates:(NSArray *)states;

- (SHPStateRepresentation *)representationForState;

- (void)exitCurrentConfigurationWithEventMessage:(SHPEventMessage *)eventMessage transitionRepresentation:(SHPTransitionRepresentation *)transitionRepresentation;
- (void)enterDefaultConfigurationWithEventMessage:(SHPEventMessage *)eventMessage transitionRepresentation:(SHPTransitionRepresentation *)transitionRepresentation;
- (void)enterDefaultConfigurationWithEventMessage:(SHPEventMessage *)eventMessage enterSelf:(BOOL)enterSelf transitionRepresentation:(SHPTransitionRepresentation *)transitionRepresentation;
- (void)enterTargetConfigurationWithTargetStates:(NSArray *)targetStates eventMessage:(SHPEventMessage *)eventMessage enterSelf:(BOOL)enterSelf historyMode:(HistoryMode)historyMode transitionRepresentation:(SHPTransitionRepresentation *)transitionRepresentation;
- (void)enterDeepHistoryWithEventMessage:(SHPEventMessage *)eventMessage transitionRepresentation:(SHPTransitionRepresentation *)transitionRepresentation;

- (NSArray *)transitionsForEvent:(SEL)event checkCondition:(BOOL)condition;

// Add transitions to other states
+ (void)onEvent:(SEL)event fromRegionStates:(NSArray *)fromRegionStates transitionTo:(NSArray *)toRegionStates;
- (void)onEvent:(SEL)event transitionToShallowHistoryOf:(SHPState *)toState;
- (void)onEvent:(SEL)event transitionToDeepHistoryOf:(SHPState *)toState;
- (void)transitionTo:(SHPState *)toState;
- (void)transitionTo:(SHPState *)toState action:(SEL)action;
- (void)transitionTo:(SHPState *)toState guard:(SHPStateChartGuard *)guard;

- (void)transitionTo:(SHPState *)toState guard:(SHPStateChartGuard *)guard reverse:(BOOL)reverse;

- (void)transitionTo:(SHPState *)toState action:(SEL)action guard:(SHPStateChartGuard *)guard;

- (void)transitionTo:(SHPState *)toState action:(SEL)action guard:(SHPStateChartGuard *)guard reverse:(BOOL)reverse;

- (void)onEvent:(SEL)event transitionTo:(SHPState *)to;
- (void)onEvent:(SEL)event transitionTo:(SHPState *)to guard:(SHPStateChartGuard *)guard;
- (void)onEvent:(SEL)event transitionTo:(SHPState *)toState guard:(SHPStateChartGuard *)guard timeout:(NSNumber *)timeout;
- (void)onEvent:(SEL)event transitionToStates:(NSArray *)toStates guard:(SHPStateChartGuard *)guard;
- (void)onTimeout:(NSNumber *)timeout transitionTo:(SHPState *)toState event:(SEL)event guard:(SHPStateChartGuard *)guard historyMode:(HistoryMode)historyMode;

// Timeouts
- (void)setupTimeoutTimers;
- (void)removeTimeoutTimers;

// Deferral
- (void)deferEvent:(SEL)event;
- (BOOL)shouldDeferEvent:(SEL)event;

// Add internal transitions
- (void)internalTransitionOnEvent:(SEL)event guard:(SHPStateChartGuard *)guard timeout:(NSNumber *)timeout;
- (void)internalTransitionOnEvent:(SEL)event;

// Representation
- (SHPStateRepresentation *)representationForState;

@end
