// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import "SHPStateChartTransition.h"
#import "SHPStateChartGuard.h"
#import "SHPOrState.h"
#import "SHPTransitionRepresentation.h"
#import "NSArrayCollections.h"


@implementation SHPStateChartTransition

- (instancetype)initWithEvent:(SEL)event fromStates:(NSArray *)fromStates toStates:(NSArray *)toStates timeout:(NSNumber *)timeout internal:(BOOL)internal guard:(SHPStateChartGuard *)guard historyMode:(HistoryMode)historyMode {
    self = [super init];
    if (self) {
        _event = event;
        _fromStates = fromStates;
        _toStates = toStates;
        _guard = guard;
        _timeout = timeout;
        _internal = internal;
        _historyMode = historyMode;
    }

    return self;
}

+ (instancetype)transitionWithEvent:(SEL)event fromStates:(NSArray *)fromStates toStates:(NSArray *)toStates timeout:(NSNumber *)timeout internal:(BOOL)internal guard:(SHPStateChartGuard *)guard historyMode:(HistoryMode)historyMode {
    return [[SHPStateChartTransition alloc] initWithEvent:event fromStates:fromStates toStates:toStates timeout:timeout internal:internal guard:guard historyMode:historyMode];
}

- (NSString *)description {
    NSMutableString *description = [NSMutableString stringWithFormat:@"<%@: ", NSStringFromClass([self class])];
    [description appendFormat:@"self.event=%p", self.event];
    [description appendFormat:@", self.fromStates=%@", self.fromStates];
    [description appendFormat:@", self.toStates=%@", self.toStates];
    [description appendFormat:@", self.guard=%@", self.guard];
    [description appendFormat:@", self.timeout=%@", self.timeout];
    [description appendFormat:@", self.internal=%d", self.internal];
    [description appendFormat:@", self.historyMode=%i", self.historyMode];
    [description appendString:@">"];
    return description;
}

- (SHPTransitionRepresentation *)representationForTransition {
    NSString *eventRepresentation = NSStringFromSelector(_event);
    NSArray *fromStatesRepresentations = [_fromStates collect:^id(SHPState *state) { return [state representationForState]; }];
    NSArray *toStatesRepresentations = [_toStates collect:^id(SHPState *state) { return [state representationForState]; }];
    id guardRepresentation = [_guard representationForGuard];
    NSString *historyModeRepresentation = [self stringForHistoryMode:_historyMode];

    SHPTransitionRepresentation *transitionRepresentation = [[SHPTransitionRepresentation alloc] initWithEvent:eventRepresentation fromStates:fromStatesRepresentations toStates:toStatesRepresentations timeout:_timeout internal:_internal guard:guardRepresentation historyMode:historyModeRepresentation];

    return transitionRepresentation;
}

- (NSString *)stringForHistoryMode:(HistoryMode)mode {
    switch (mode) {
        case HistoryModeNone:
            return @"None";
        case HistoryModeShallow:
            return @"Shallow";
        case HistoryModeDeep:
            return @"Deep";
        default:
            return @"<UNKNOWN HISTORY MODE>";
    }
}

@end
