// This software is licensed under GNU GLP version 3. Please read the file licence.txt for the complete licence text.
// 
// Copyright (c) 2014, Peter Gammelgaard, Philip Bruce & Shape A/S. All rights reserved.

#import "SHPMessageRouter.h"
#import "SHPMessageTracingComponent.h"
#import "SHPEventMessage.h"
#import "SHPTracingMessage.h"


@implementation SHPMessageTracingComponent {
    NSMutableDictionary *_traces;
}

- (id)init {
    self = [super init];
    if (self) {
        _messageInputPort = [self inputPortWithProtocol:@protocol(SHPMessageRouterMonitorPortProtocol) name:@"Trace"];
        _commandInputPort = [self inputPortWithProtocol:@protocol(SHPMessageTracingCommandProtocol) name:@"Command"];
        _traces = [NSMutableDictionary new];
    }

    return self;
}

- (void)didProcessMessage:(SHPEventMessage *)message {
    NSString *identifier = message.traceIdentifier;
    NSMutableDictionary *trace = [_traces objectForKey:identifier];
    if(!trace) {
        trace = [NSMutableDictionary new];
    }

    SHPTracingMessage *parentMessage = [trace objectForKey:message.parentMessage.identifier];
    SHPTracingMessage *tracingMessage = [[SHPTracingMessage alloc] initWithMessage:message parentTracingMessage:parentMessage];

    [trace setObject:tracingMessage forKey:message.identifier];

    [_traces setObject:trace forKey:identifier];
}

- (void)printTraces {
    NSArray *traces = _traces.allValues;

    [traces enumerateObjectsUsingBlock:^(NSDictionary *traceDict, NSUInteger idx, BOOL *stop) {
        NSArray *trace = traceDict.allValues;
        NSArray *sortedTrace;
        sortedTrace = [trace sortedArrayUsingComparator:^NSComparisonResult(SHPTracingMessage *messageTrace1, SHPTracingMessage *messageTrace2) {
            return (NSComparisonResult) (messageTrace1.message.timestamp>messageTrace2.message.timestamp);
        }];
        NSLog(@"%@",sortedTrace);
    }];


}


@end